# -*- coding:utf-8 -*-
# @time:2023/4/198:53
# @author:LX
# @file:animation.py
# @software:PyCharm
from PyQtGuiLib.header import (
    QEasingCurve
)
from typing import Callable,Any,TypeVar,List,Union,Tuple
from PyQtGuiLib.animation.PropertyAnimation import ParallelAnimationGroup,SequentialAnimationGroup,PropertyAnimation


AnimationGroup = TypeVar("AnimationGroup",ParallelAnimationGroup,SequentialAnimationGroup)
'''
    动画框架重写
'''


class Animation:
    Parallel = 1
    Sequential = 2

    STOP = 0
    PAUSE = 1
    START = 2

    def __init__(self,):
        self.__startMode = SequentialAnimationGroup()  # 默认串行
        self.__anis = []  # type:List[PropertyAnimation]
        self.isBuilt = False

    def allAnins(self)->List[PropertyAnimation]:
        return self.__anis

    def setStartMode(self,mode:AnimationGroup):
        if Animation.Parallel == mode:
            self.__startMode = ParallelAnimationGroup()
        elif Animation.Sequential == mode:
            self.__startMode = SequentialAnimationGroup()
        else:
            raise Exception("No such mode!")

    def startMode(self) -> int:
        if isinstance(self.__startMode,SequentialAnimationGroup):
            return Animation.Sequential
        else:
            return Animation.Parallel

    def startModeObj(self)->AnimationGroup:
        return self.__startMode

    def addAni(self,
               sv:Any,ev:Any,
               atv:Union[List[Any],List[Tuple[float,Any]]]=None,
               targetObj = None,
               duration:int = 1200,
               propertyName:bytes = b"",
               loopCount:int = 1,
               easingCurve:QEasingCurve = None,
               selector:str = None,
               courseFunc:Callable = None,
               finishedCall:Callable = None,
               **kwargs):
        '''


        :param sv:
        :param ev:
        :param atv:
        :param targetObj:
        :param duration:
        :param propertyName:
        :param loopCount:
        :param easingCurve:
        :param selector: QSS选择器,注意:如果不是为了样式的动画,请不要设置该属性,避免多余的内存开销
        :param courseFunc:
        :param finishedCall:
        :param kwargs:
        :return:
        '''
        if targetObj and propertyName:
            ani = PropertyAnimation()
            ani.setTargetObject(targetObj)
            if selector:
                # 这一段保证了,多个QSS被修改时,最终的结果是一致的
                obj = self.findTargetObject(targetObj)
                if obj:
                    qss = obj.qss()
                    if qss:ani.setSelector(selector,qss)
                    else:ani.setSelector(selector)
                else:
                    ani.setSelector(selector)
            ani.setPropertyName(propertyName)
        elif targetObj and courseFunc:
            ani = PropertyAnimation()
            ani.setTargetObject(targetObj)
            ani.courseFunc(courseFunc)
        elif targetObj:
            ani = PropertyAnimation()
            ani.setTargetObject(targetObj)
        elif courseFunc:
            ani = PropertyAnimation()
            ani.courseFunc(courseFunc)
        else:
            raise Exception('''
            Parameter passing error, when there is targetObj, targetObj and propertyName must be valid at the same time, 
            or targetObj and courseFunc must be valid at the same time, or, when there is no targetObj, 
            courseFunc is a required parameter.
            ''')

        if atv:ani.setStartEndValue(sv,ev,atv)
        else:ani.setStartEndValue(sv, ev)

        if easingCurve:
            ani.setEasingCurve(easingCurve)

        ani.setDuration(duration)
        ani.setLoopCount(loopCount)
        if finishedCall:
            ani.finishedCall(finishedCall)
        self.__anis.append(ani)

    def addSeriesAni(self,
                     sv: Any, ev: Any,evs:List[Any],
                     atv: Union[List[Any], List[Tuple[float, Any]]] = None,
                     targetObj = None,
                     duration: int = 1200,
                     propertyName: bytes = b"",
                     loopCount: int = 1,
                     easingCurve: QEasingCurve = None,
                     selector: str = None,
                     courseFunc: Callable = None,
                     finishedCall: Callable = None,
                     **kwargs):
        '''

            使用该该函数必须将使用 setStartMode()将动画模式设置为串型(默认也是串型)

        :param sv:
        :param ev:
        :param evs: 连续变化值列表(必须是同一类型)
        :param atv:
        :param targetObj:
        :param duration:
        :param propertyName:
        :param loopCount:
        :param easingCurve:
        :param selector:
        :param courseFunc:
        :param finishedCall:
        :param kwargs:
        :return:
        '''
        if self.startMode() != Animation.Sequential:
            raise Exception("Sequential Animation is \"Animation.Sequential\" mode.")

        if not evs:
            raise Exception("The \"evs\" continuous change item should not be empty!")

        # 首先创建第一个动画
        self.addAni(sv,ev,atv,targetObj,duration,propertyName,
                    loopCount,easingCurve,selector,courseFunc,None,**kwargs)

        first_value = ev
        two_value = None

        for _ev in evs[:-1]:
            two_value = _ev
            self.addAni(first_value, two_value, atv, targetObj, duration, propertyName,
                        loopCount, easingCurve, selector, courseFunc, None, **kwargs)
            first_value = two_value

        if finishedCall:
            self.addAni(evs[-2], evs[-1],atv,targetObj,duration,propertyName,
                        loopCount,easingCurve,selector,courseFunc,finishedCall,**kwargs)
        else:
            self.addAni(evs[-2], evs[-1], atv, targetObj, duration, propertyName,
                        loopCount, easingCurve, selector, courseFunc, None, **kwargs)

    # 查找一个控件对象
    def findTargetObject(self,targetObj:Any)->Union[PropertyAnimation,None]:
        for obj in self.__anis:
            if targetObj is obj.targetObject():
                return obj
        return None

    def builtAni(self):
        # 该方法只会生效一次
        if not self.isBuilt:
            self.startModeObj().addAnimations(self.allAnins())
            self.isBuilt = True

    def start(self,isBuilt:bool=True):
        if isBuilt:
            self.builtAni()
        if not self.startModeObj().state():
            self.startModeObj().start()

    def state(self) -> int:
        return self.startModeObj().state()

    def resume(self):
        self.startModeObj().resume()

    def paused(self):
        self.startModeObj().pause()

    def stop(self):
        self.startModeObj().stop()


from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    CustomStyle,
    qt,
    Qt,
    QWidget,
    QMouseEvent,
    QPaintEvent,
    QPainter,
    QLinearGradient,
    QPen,
    QBrush,
    QPoint,
    QColor,
    QEvent
)
import re
import json


'''
    所有窗口的父类
'''

class WidgetABC(QWidget,CustomStyle):
    '''
        支持的自定义样式
    qproperty-radius  --> 圆角  Eg: 7
    qproperty-backgroundColor  --> 背景颜色 Eg: rgba(165, 138, 255,200)
    qproperty-borderWidth --> 边的宽度 Eg: 1
    qproperty-borderStyle --> 边框的风格 Eg: solid
    qproperty-borderColor --> 边框颜色 Eg: rgba(0,100,255,255)
    qproperty-border   --> 边框样式 Eg: "3 solid rgba(0,100,255,255)"
    qproperty-linearDirection; --> 线性渐变的方向 Eg: "LR"
        LR: 左->右
        RL: 右->左
        UD: 上->下
        DU: 下->上
        LRANG: 左上角->右下角
        RLANG: 右下角->左上角
        UDANG: 右上角->左下角
        DUANG: 左下角->右上角
        自定义: [0,0,100,100]或者[0,0,w,h]  这里的 w,h 代只窗口当前的宽和高
    qproperty-linearColor --> 线性渐变色 Eg: "rgba(142, 144, 69, 255) rgba(176, 184, 130, 255) rgba(255, 255, 255, 255)"
    qproperty-linear --> 线性渐变
        Eg: "LR rgba(142, 144, 69, 255) rgba(176, 184, 130, 255) rgba(130, 184, 130, 255)";
        Eg: "[0,0,w,h] rgba(142, 144, 69, 255) rgba(176, 184, 130, 255) rgba(130, 184, 130, 255)";

    '''
    Top = "top"
    Down = "Down"
    Left = "Left"
    Right = "Right"

    # 渐变的方向
    G_Vertical = "vertical"
    G_Horizontal = "horizontal"

    def __init__(self,*args,**kwargs):
        # 窗口操作限制器,当前窗口作为子窗口时,触发
        self.__RestrictedOperation = False if args else True
        if PYQT_VERSIONS in ["PySide6","PySide2"]:
            QWidget.__init__(self,*args, **kwargs)
            CustomStyle.__init__(self,*args, **kwargs)
        else:
            super().__init__(*args,**kwargs)
        self.resize(700,500)

        self.setAttribute(qt.WA_TranslucentBackground)
        self.setWindowFlags(qt.FramelessWindowHint | qt.Widget)
        self.setMouseTracking(True)  # 开启鼠标跟踪

        # 用于窗口的移动属性
        self.scope = 10  # 检测鼠标是否在边缘按下的范围
        self.pressDirection = []  # 记录鼠标点击的边的方向
        self.pressPos = QPoint(0, 0)  # 鼠标按下时的位置
        self.pressState = False  # 按下状态
        self.movePressState = False  # 记录移动时是否时按下的状态

        # 渐变色
        self.__is_e_gcolor = False  # 是否启用渐变色
        self.w_g_direction = "horizontal"  # 渐变方向

        # 操作限制
        self.installEventFilter(self)

    # 设置窗口模态
    def setWinModality(self,b:bool):
        self.setWindowModality(Qt.ApplicationModal)

    # 设置 限制操作
    def setRestrictedOperation(self,b:bool):
        self.__RestrictedOperation = b

    # 设置 背景颜色渐变启动
    def setEnableGColor(self,b:bool):
        self.__is_e_gcolor = b

    def isEnableGColor(self)->bool:
        return self.__is_e_gcolor

    # 边缘检测
    def isEdge(self,pos: QPoint):
        x, y = pos.x(), pos.y()
        w, h = self.width(), self.height()

        if x <= self.scope and x >= 0:
            self.pressDirection.append(WidgetABC.Left)
        if w - x <= self.scope:
            self.pressDirection.append(WidgetABC.Right)
        if y >= 0 and y <= self.scope:
            self.pressDirection.append(WidgetABC.Top)
        if h - y <= self.scope:
            self.pressDirection.append(WidgetABC.Down)

        if self.pressDirection:
            return True

    # 更新鼠标的图标
    def updateCursor(self, pos: QPoint):
        x, y = pos.x(), pos.y()
        w, h = self.width(), self.height()
        '''
            先判断四角,再判断四边
        '''
        if self.__RestrictedOperation is False:
            if self.pressDirection:
                return True
            return

        if (x <= self.scope and x >= 0) and (y >= 0 and y <= self.scope) \
                or (w - x <= self.scope) and (h - y <= self.scope):
            self.setCursor(qt.SizeFDiagCursor)
        elif (w - x <= self.scope) and (y >= 0 and y <= self.scope) \
                or (x <= self.scope and x >= 0) and (h - y <= self.scope):
            self.setCursor(qt.SizeBDiagCursor)
        elif (x <= self.scope and x >= 0) or (w - x <= self.scope):
            self.setCursor(qt.SizeHorCursor)
        elif (y >= 0 and y <= self.scope) or (h - y <= self.scope):
            self.setCursor(qt.SizeVerCursor)
        else:
            self.setCursor(qt.ArrowCursor)

        if self.pressDirection:
            return True

    # 扩展边
    def expandEdge(self, pos: QPoint):
        if not self.pressState:
            return

        x, y = self.pos().x(), self.pos().y()

        for direction in self.pressDirection:
            if direction == WidgetABC.Right:
                if WidgetABC.Down not in self.pressDirection:
                    distance = pos - self.pressPos
                    distance_ = self.width() + distance.x()
                    self.resize(distance_, self.height())
                    self.pressPos = QPoint(distance_, 0)
            if direction == WidgetABC.Down:
                distance = pos - self.pressPos
                distance_ = self.height() + distance.y()
                if WidgetABC.Right in self.pressDirection:
                    distance_w = self.width() + distance.x()
                    self.resize(distance_w, distance_)
                    self.pressPos = QPoint(distance_w, distance_)
                else:
                    self.resize(self.width(), distance_)
                    self.pressPos = QPoint(0, distance_)
            if direction == WidgetABC.Left:
                distance_ = x + pos.x()
                self.resize(self.width() - pos.x(), self.height())
                self.move(distance_, y)
            if direction == WidgetABC.Top:
                distance_ = y + pos.y()
                self.resize(self.width(), self.height() - pos.y())
                if WidgetABC.Left in self.pressDirection:
                    x = x + pos.x()
                self.move(x, distance_)

    # 绘制背景
    def drawBackgroundColor(self,painter:QPainter):
        # 绘制背景
        if self.__is_e_gcolor:
            # 绘制渐变色
            temp_linearDirection = self.get_linearDirection()
            linearDirection_dict = {
                "w": str(self.width()),
                "h": str(self.height())
            }
            for c in re.findall(r"[a-z]", temp_linearDirection):
                v = linearDirection_dict.get(c, None)
                if v:
                    temp_linearDirection = temp_linearDirection.replace(c, v)

            dir_value = json.loads(temp_linearDirection)
            gradient = QLinearGradient(*dir_value)

            # 分析颜色,并均分
            colors = []
            color_list = re.findall(r"rgba\(.+?\)", self.get_linearColor())
            color_number = len(color_list)
            if color_number > 2:
                paragraph_v = 1 / (color_number - 1)
            else:
                paragraph_v = 0
            v = 0  # 渐变的取值
            for cstr in color_list:
                v_str = re.findall(r"\d{1,3}", cstr)
                colors.append([v, QColor(*[int(i) for i in v_str])])
                if color_number > 2:
                    v += paragraph_v
            # 修改最后一个渐变值
            colors[-1][0] = 1
            for v, c in colors:
                gradient.setColorAt(v, c)
            bgcolor = gradient
            painter.setBrush(gradient)
        else:
            # 绘制纯色背景
            # print("-->",self.get_backgroundColor())
            bgcolor = QBrush(self.get_backgroundColor())

        painter.setBrush(bgcolor)
        # 绘制
        rect_ = self.rect()
        rect_.setX(rect_.x() + self.get_margin())
        rect_.setY(rect_.y() + self.get_margin())
        rect_.setWidth(rect_.width() - self.get_margin())
        rect_.setHeight(rect_.height() - self.get_margin())
        painter.drawRoundedRect(rect_, self.get_radius(), self.get_radius())

    def eventFilter(self, obj: 'QObject', event:QEvent) -> bool:
        # 做为子窗口时,限制事件,event.MouseMove
        if self.__RestrictedOperation is False and event.type() in [event.KeyRelease,event.KeyPress,event.MouseMove]:
            return True
        else:
            return super().eventFilter(obj,event)

    def mousePressEvent(self, e: QMouseEvent) -> None:
        if self.isEdge(e.pos()):
            self.pressState = True
            self.pressPos = e.pos()
        elif e.button() == qt.LeftButton:  # 处理窗口移动
            self.movePressState = True

            if PYQT_VERSIONS in ["PyQt5", "PySide2", "PySide6"]:
                old_pos = e.globalPos()
            elif PYQT_VERSIONS == "PyQt6":
                old_pos = e.globalPosition().toPoint()
            else:
                old_pos = QPoint(0,0)

            self.pressPos = old_pos - self.pos()

            # self.setCursor(qt.OpenHandCursor)

    def mouseReleaseEvent(self,e:QMouseEvent) -> None:
        self.pressDirection.clear()
        self.pressPos = QPoint(0, 0)
        self.pressState = False
        self.movePressState = False

    def mouseMoveEvent(self, e: QMouseEvent) -> None:
        if self.movePressState:
            if PYQT_VERSIONS in ["PyQt5", "PySide2", "PySide6"]:
                old_pos = e.globalPos()
            elif PYQT_VERSIONS == "PyQt6":
                old_pos = e.globalPosition().toPoint()
            else:
                old_pos = QPoint(0, 0)
            self.move(old_pos - self.pressPos)

        self.updateCursor(e.pos())
        self.expandEdge(e.pos())

    def paintEvent(self, event:QPaintEvent) -> None:
        painter = QPainter(self)
        painter.setRenderHints(qt.Antialiasing | qt.SmoothPixmapTransform)  # 抗锯齿

        # 透明原本边框
        op = QPen()
        op.setColor(qt.transparent)
        painter.setPen(op)

        # 绘制背景
        self.drawBackgroundColor(painter)

        # 绘制边框
        op.setWidth(self.get_borderWidth())
        op.setColor(self.get_borderColor())
        op.setStyle(self.get_borderStyle())
        painter.setPen(op)

        painter.drawRoundedRect(self.rect(),self.get_radius(),self.get_radius())

        painter.end()

if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = WidgetABC()
    win.setStyleSheet('''
WidgetABC{
 qproperty-radius:5;
/*qproperty-backgroundColor:rgba(255,0,2,255);*/
/*qproperty-borderColor:rgba(255,0,2,255);*/
}
    ''')
    win.show()

    if PYQT_VERSIONS in ["PyQt6", "PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2023/5/2715:31
# @author:LX
# @file:paletteToolUI.py
# @software:PyCharm
from PyQtGuiLib.header import (
    QApplication,
    QWidget,
    QVBoxLayout,
    QHBoxLayout,
    QPushButton,
    Signal,
    QLabel,
    QLineEdit,
    QPainter,
    QSpacerItem,
    QSizePolicy,
    Qt,
    QFormLayout,
    QStackedWidget,
    QColor,
    qt,
    QTimer,
    QCursor,
    QPoint
)

from PyQtGuiLib.core.switchButtons.swButton import SwitchButton
from PyQtGuiLib.styles.styleAnalysis import QssStyleAnalysis
from PyQtGuiLib.core.slideShow import SlideShow
from PyQtGuiLib.styles.superPainter.superPainter import SuperPainter, VirtualObject


Handle_pure = "pure"
Handle_Linear = "linear"
Handle_Radial = "radial"
Handle_Conical = "conical"

G_Mode_Pad = "pad"
G_Mode_Repeat = "repeat"
G_Mode_Reflect = "reflect"


class MaskWidget(QWidget):
    clickColor = Signal(QColor)

    def __init__(self):
        super().__init__()
        self.size = 150
        self.half_size = self.size//2

        self.resize(self.size, self.size)
        self.setWindowFlags(qt.FramelessWindowHint)
        self.setWindowOpacity(0.01)

        self.setCursor(Qt.CrossCursor)

        self.timer = QTimer()
        self.timer.timeout.connect(self.updatePos)

    def updatePos(self):
        self.move(QCursor.pos()-QPoint(self.half_size, self.half_size))
        screen = QApplication.primaryScreen()
        if screen is not None:
            pos = QCursor.pos()
            pixmap = screen.grabWindow(0, pos.x(), pos.y(), 1, 1)
            color = pixmap.toImage().pixelColor(0, 0)
            self.clickColor.emit(color)

    def show(self) -> None:
        self.timer.start(15)
        super().show()

    def mousePressEvent(self, e) -> None:
        self.timer.stop()
        self.close()
        super().mousePressEvent(e)


# 纯色,渐变,径向,辐射(角度)渐变 的 切换 部件类
class StateColorActionBarUI(QWidget):
    clicked = Signal(str)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.core_hlay = QHBoxLayout(self)
        self.core_hlay.setAlignment(Qt.AlignLeft)

        self.pure_color = QPushButton()
        self.linear_color_btn = QPushButton()
        self.radial_color_btn = QPushButton()
        self.conical_color_btn = QPushButton()

        self.pure_color.setToolTip("纯色")
        self.linear_color_btn.setToolTip("线性渐变")
        self.radial_color_btn.setToolTip("径向渐变")
        self.conical_color_btn.setToolTip("角度渐变")

        for btn in [self.pure_color, self.linear_color_btn, self.radial_color_btn, self.conical_color_btn]:
            btn.setFixedSize(24, 24)
            self.core_hlay.addWidget(btn)

        self.pure_color.setObjectName("pure_color")
        self.linear_color_btn.setObjectName("linear_color")
        self.radial_color_btn.setObjectName("radial_color")
        self.conical_color_btn.setObjectName("conical_color")

        self.defaultStyle()

        self.pure_color.clicked.connect(lambda :self.__click_event(Handle_pure))
        self.linear_color_btn.clicked.connect(lambda :self.__click_event(Handle_Linear))
        self.radial_color_btn.clicked.connect(lambda :self.__click_event(Handle_Radial))
        self.conical_color_btn.clicked.connect(lambda :self.__click_event(Handle_Conical))

    def __click_event(self,state):
        for name in [Handle_pure,Handle_Linear,Handle_Radial,Handle_Conical]:
            if state == name:
                self.qss.selector("#{}_color".format(name)).updateAttr("border","2px solid white")
            else:
                self.qss.selector("#{}_color".format(name)).updateAttr("border","2px solid black")
        self.clicked.emit(state)

    def defaultStyle(self):
        self.qss = QssStyleAnalysis(self)
        self.qss.setQSS('''
#pure_color,#linear_color,#radial_color,#conical_color{
border-radius:12px;
border:2px solid black;
}
#pure_color:hover,#linear_color:hover,#radial_color:hover,#conical_color:hover{
border:3px solid white;
}
#pure_color:pressed,#linear_color:pressed,#radial_color:pressed,#conical_color:pressed{
border:2px solid black;
}
#pure_color{
border:2px solid white;
background-color: rgb(255, 170, 0);
}
#linear_color{
background-color:qlineargradient(spread:pad,x1:0.002,y1:0.457,x2:0.957,y2:0.463,stop:0.49148936170212765 rgba(217, 155, 9, 255),stop:0.5893617021276596 rgba(173, 166, 147, 255));
}
#radial_color{
background-color:qradialgradient(spread:pad,cx:0.436,cy:0.483,radius:0.462,fx:0.47,fy:0.533,stop:0.49148936170212765 rgba(217, 155, 9, 255),stop:0.5893617021276596 rgba(173, 166, 147, 255));
}
#conical_color{
background-color:qconicalgradient(cx:0.448, cy:0.42, angle:120 stop:0.49148936170212765 rgba(217, 155, 9, 255),stop:0.5893617021276596 rgba(173, 166, 147, 255));
}
        ''')

    def paintEvent(self, e) -> None:
        painter = QPainter(self)

        painter.setRenderHint(QPainter.Antialiasing,True)

        painter.end()


# 切换渐变模式UI 部件类
class GradientModeActionBarUI(QWidget):
    clicked = Signal(str)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.core_hlay = QHBoxLayout(self)
        self.core_hlay.setAlignment(Qt.AlignLeft)

        self.pad_btn = QPushButton()
        self.repeat_btn = QPushButton()
        self.reflect_btn = QPushButton()

        for btn in [self.pad_btn,self.repeat_btn,self.reflect_btn]:
            btn.setFixedSize(48,24)
            self.core_hlay.addWidget(btn)

        self.pad_btn.setObjectName("pad_mode")
        self.repeat_btn.setObjectName("repeat_mode")
        self.reflect_btn.setObjectName("reflect_mode")

        self.defaultStyle()

        self.pad_btn.clicked.connect(lambda :self.__click_event(G_Mode_Pad))
        self.repeat_btn.clicked.connect(lambda :self.__click_event(G_Mode_Repeat))
        self.reflect_btn.clicked.connect(lambda :self.__click_event(G_Mode_Reflect))

    def __click_event(self,mode):
        for name in [G_Mode_Pad,G_Mode_Repeat,G_Mode_Reflect]:
            if mode == name:
                self.qss.selector("#{}_mode".format(name)).updateAttr("border","2px solid white")
            else:
                self.qss.selector("#{}_mode".format(name)).updateAttr("border","2px solid black")
        self.clicked.emit(mode)

    def defaultStyle(self):
        self.qss = QssStyleAnalysis(self)
        self.qss.setQSS('''
#pad_mode:hover,#repeat_mode:hover,#reflect_mode:hover{
border:2px solid white;
}
#pad_mode,#repeat_mode,#reflect_mode,#pad_mode:pressed,#repeat_mode:pressed,#reflect_mode:pressed{
border:2px solid back;
border-radius:3px;
}
#pad_mode{
border:2px solid white;
background-color: qlineargradient(spread:pad, x1:0.001, y1:0.471, x2:1, y2:0.482955, stop:0 rgba(232, 200, 97, 255), stop:1 rgba(174, 174, 174, 255));
}
#repeat_mode{
background-color:qlineargradient(spread:repeat, x1:0.33, y1:0.482364, x2:0.63, y2:0.477273, stop:0 rgba(232, 200, 97, 255), stop:1 rgba(174, 174, 174, 255));
}
#reflect_mode{
background-color:qlineargradient(spread:reflect, x1:0.554, y1:0.475, x2:0.63, y2:0.477273, stop:0 rgba(232, 200, 97, 255), stop:1 rgba(174, 174, 174, 255));
}
        
        
        ''')


# 其他部件类
class ItActionBarUI(QWidget):
    switchClicked = Signal(bool)
    strawColored = Signal(QColor)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.core_hlay = QHBoxLayout(self)
        self.core_hlay.setAlignment(Qt.AlignLeft)

        # 纯色预览方块
        self.color_view = QLabel()
        self.color_view.setFixedSize(24,24)
        # 吸管功能按钮
        self.straw_btn = QPushButton()
        self.straw_btn.setText("吸")
        self.straw_btn.setToolTip("颜色吸管")
        self.straw_btn.setFixedSize(24, 24)
        # 句柄开关按钮
        self.hand_btn = SwitchButton()
        self.hand_btn.setToolTip("渐变句柄 显示/隐藏 开关")
        self.hand_btn.setShape(SwitchButton.Shape_Square)
        self.hand_btn.clicked.connect(self.switchClicked.emit)
        self.hand_btn.setFixedSize(50, 24)
        # 六十进制颜色显示
        self.hex_btn = QPushButton()
        self.hex_btn.setToolTip("点击可以复制")
        self.hex_btn.setFixedSize(65, 24)
        self.hex_btn.setText("#00ff00")

        self.color_view.setObjectName("color_view")
        self.straw_btn.setObjectName("straw_btn")
        self.hand_btn.setObjectName("hand_btn")
        self.hex_btn.setObjectName("hex_btn")

        self.core_hlay.addWidget(self.color_view)
        self.core_hlay.addWidget(self.straw_btn)
        self.core_hlay.addWidget(self.hand_btn)
        self.core_hlay.addWidget(self.hex_btn)

        self.straw_btn.clicked.connect(self.straw)

        self.defaultStyle()

        self.myEvent()

    # 复制颜色到鼠标
    def copyColorButton(self)->str:
        copy_color = QApplication.clipboard()
        copy_color.setText(self.hex_btn.text())
        # 缺少通知动画
        # .....

    # 吸管功能
    def straw(self):
        if not hasattr(self,"straw_widget"):
            self.straw_widget = MaskWidget()
            self.straw_widget.clickColor.connect(self.strawColored.emit)
            self.straw_widget.clickColor.connect(self.updateHexView)
        self.straw_widget.show()

    def defaultStyle(self):
        self.setStyleSheet('''
#color_view{
border-radius:2px;
background-color: rgb(88, 142, 128);
}
#straw_btn{
font: 10pt "等线";
background-color: rgb(20, 134, 79);
color: rgb(232, 232, 232);
border:1px solid rgb(85, 170, 127);
border-radius:2px;
}
#straw_btn:pressed{
background-color:rgb(14, 95, 56)
}
#straw_btn:hover{
border-color:white;
}
#hex_btn:hover{
font-size:11pt;
}
#hex_btn,#hex_btn:pressed{
border:none;
color:white;
border-radius:2px;
border-bottom-left-radius:2px;
background-color: rgb(20, 134, 79);
font: 10pt "等线";
}
        ''')

    def updateViewLabel(self,color):
        self.color_view.setStyleSheet('''
        background-color:rgba({},{},{},{});
        '''.format(*color.getRgb()))

    def updateHexColor(self,color):
        self.hex_btn.setText(color.name())

    def updateHexView(self,color):
        self.updateViewLabel(color)
        self.updateHexColor(color)

    def myEvent(self):
        self.hex_btn.clicked.connect(self.copyColorButton)


# -----------------------------

# 纯色的操作台
class PureColorOperationUI(QWidget):
    clickColor = Signal()

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        self.vlay = QVBoxLayout(self)
        self.vlay.setAlignment(Qt.AlignTop)

        self.labels = [QLabel() for _ in range(4)]
        self.lines = [QLineEdit() for _ in range(4)]

        for i,text in enumerate(["红(R)","绿(G)","蓝(B)","透(A)"]):
            self.labels[i].setText(text)
            self.lines[i].setText("255")

        self.flay = QFormLayout()
        self.flay.setSpacing(3)
        for i in range(4):
            self.flay.addRow(self.labels[i],self.lines[i])
            self.lines[i].setReadOnly(True)

        self.colorButton = QPushButton("获取颜色")
        self.colorButton.setFixedSize(90, 30)
        self.colorButton.setObjectName("colorButton")
        self.flay.setWidget(self.flay.rowCount(), QFormLayout.SpanningRole, self.colorButton)

        self.vlay.addLayout(self.flay)

        self.colorButton.clicked.connect(self.clickColor.emit)

        self.defaultStyle()

    def updateRGB(self,color):
        self.lines[0].setText(str(color.red()))
        self.lines[1].setText(str(color.green()))
        self.lines[2].setText(str(color.blue()))

    def updateAlpha(self, v:int):
        self.lines[3].setText(str(v))

    def defaultStyle(self):
        self.setStyleSheet('''
QLabel{
color: rgb(108, 210, 171);
font: 11pt "等线";
}
QLineEdit{
border:none;
background-color: rgba(23, 154, 93,100);
color: rgb(209, 209, 209);
}
#colorButton:hover{
background-color: rgb(74, 120, 108);
}
#colorButton,#colorButton:pressed{
border-radius:3px;
background-color: rgb(88, 142, 128);
color: rgb(255, 255, 255);
font: 11pt "等线";
}
        ''')


# 渐变操作台
class GradientOperationUI(QWidget):
    clickQSS = Signal()

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        self.vlay = QVBoxLayout(self)
        self.vlay.setAlignment(Qt.AlignTop)

        self.qss = QPushButton()
        self.qss.setObjectName("qss")
        self.qss.setText("获取QSS")
        self.qss.setFixedSize(90,30)

        self.qss.clicked.connect(self.clickQSS.emit)

        self.vlay.addWidget(self.qss)

        self.defaultStyle()

    def defaultStyle(self):
        self.setStyleSheet('''
#qss{
border-radius:3px;
background-color: rgb(88, 142, 128);
color: rgb(255, 255, 255);
font: 11pt "等线";
}
#qss:hover{
background-color: rgb(74, 120, 108);
}
        ''')


class LinearOperationUI(GradientOperationUI):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)


class RadialOperationUI(GradientOperationUI):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)


class ConicalOperationUI(GradientOperationUI):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)


class PaletteToolsUI(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(600, 400)

        self.setWindowTitle("Palette")
        self.setObjectName("widget")

        self.core_vlay = QVBoxLayout(self)

        self.createActionBarUI()
        self.createBodyUI()

        self.defaultStyle()

    def createActionBarUI(self):
        self.action_top_hlay = QHBoxLayout()

        # 主要小部件
        self.state_action_bar = StateColorActionBarUI()
        self.gm_action_bar = GradientModeActionBarUI()
        self.action_top_hlay.addWidget(self.state_action_bar)
        self.action_top_hlay.addWidget(self.gm_action_bar)

        # 弹簧 区别左右
        self.__spacer = QSpacerItem(0, 0, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.action_top_hlay.addItem(self.__spacer)

        # 其他部件
        self.it_action_bar = ItActionBarUI()
        self.action_top_hlay.addWidget(self.it_action_bar)

        self.core_vlay.addLayout(self.action_top_hlay)

    def createBodyUI(self):
        self.body_hlay = QHBoxLayout()
        self.body_hlay.setSpacing(0)

        # 绘制区域
        self.body_st_widget = SlideShow()
        self.body_st_widget.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Expanding)
        self.body_st_widget.setButtonsHide(True)

        # 操作区域
        self.body_op_widget = QStackedWidget()
        self.body_op_widget.setFixedWidth(100)

        self.body_hlay.addWidget(self.body_st_widget)
        self.body_hlay.addWidget(self.body_op_widget)
        self.core_vlay.addLayout(self.body_hlay)

    def addColorWidget(self,widget:QWidget):
        self.body_st_widget.addWidget(widget)

    def addOperationWidget(self,widget:QWidget):
        self.body_op_widget.addWidget(widget)

    def setCurrentIndex(self,g_type):
        state_dict = {
            Handle_pure:0,
            Handle_Linear:1,
            Handle_Radial:2,
            Handle_Conical:3
        }
        i = state_dict[g_type]
        self.body_st_widget.setCurrentIndex(i)
        self.body_op_widget.setCurrentIndex(i)

    def myEvent(self):
        pass

    def defaultStyle(self):
        self.setStyleSheet('''
*{
    font-family:"等线";
}
#widget{
    background-color:#0b4a2d;
}
''')



# -*- coding:utf-8 -*-
from PyQtGuiLib.header import (
    QPainter,
    QPainterPath,
    QPen,
    QBrush,
    QFont,
    qt,
    Qt,
    QColor,
    QPoint,
    QRect,
)
import typing

'''
        drawRect 有(x,y)
        drawRoundedRect  有(x,y)
        drawLine 有(x,y)
        drawLines  
        drawArc  有(x,y)
        drawEllipse  有(x,y)
        drawPoint  有(x,y)
        drawPoints 
        drawText 有(x,y)
        drawStaticText 
        drawPolygon 
        drawPie 有(x,y)
        drawPicture 有(x,y)
        drawChord 有(x,y)
        drawRects 
        drawPolyline 
        drawPath 
        drawConvexPolygon 
        drawGlyphRun 
        drawPixmap 有(x,y)
        drawImage 有(x,y)
        drawPixmapFragments 
        drawTiledPixmap 有(x,y)
'''
RECT_TYPES = ["drawRect","drawRoundedRect","drawLine","drawArc","drawEllipse","drawPoint",
                "drawText","drawPie","drawPicture","drawChord","drawPixmap","drawImage","drawTiledPixmap"]


# 图形分类函数
def patternClassification(name:str):
    '''
        返回参数含义:
            Rect 表示前两个参数是x,y,可以直接进行移动操作

    :param name:
    :return:
    '''
    if name in RECT_TYPES:
        return "Rect"


class VirtualObject:
    def __init__(self,vobj_name:str,**kwargs):
        self.__vobj_name = vobj_name
        self.__vir_attr = kwargs # 虚拟属性

        self.__isHide = False

    def setHide(self,b:bool):
        self.__isHide = b

    def isHide(self)->bool:
        return self.__isHide

    def type(self)->str:
        return self.getVirtualFunc().__name__

    def virName(self) -> str:
        return self.__vobj_name

    def getVirtualObjectAttr(self) -> dict:
        return self.__vir_attr

    def getVirtualObjectAttrValue(self, key: str):
        return self.getVirtualObjectAttr()[key]

    def getVirtualFunc(self) -> typing.Callable:
        return self.getVirtualObjectAttr()["func"]

    def getVirtualArgs(self) -> tuple:
        return self.getVirtualObjectAttr()["args"]

    def getVirtualOpenAttr(self) -> dict:
        return self.getVirtualObjectAttr()["openAttr"]

    def getVirtualBrushAttr(self)->dict:
        return self.getVirtualObjectAttr()["brushAttr"]

    def updateArgs(self,*args):
        if args:
            self.getVirtualObjectAttr()["args"] = args

    def updateOpenAttr(self,openAttr:dict):
        if openAttr:
            self.getVirtualObjectAttr()["openAttr"].update(openAttr)

    def updateBrushAttr(self,brushAttr:dict):
        if brushAttr:
            self.getVirtualObjectAttr()["brushAttr"].update(brushAttr)

    def updateDraw(self,*args,**kwargs):
        self.updateArgs(self.virtualObjName(),*args)
        openAttr = kwargs.get("openAttr", None)
        brushAttr = kwargs.get("brushAttr", None)
        self.updateOpenAttr(self.virtualObjName(),openAttr)
        self.updateBrushAttr(self.virtualObjName(),brushAttr)

    # ----------------------------------
    def __isRect(self) ->bool:
        if patternClassification(self.type()) == "Rect":
            return True
        return False
    
    def getRect(self)->list:
        return self.getVirtualArgs()[:4]

    def getPos(self)->list:
        return self.getVirtualArgs()[:2]

    def getSize(self)->list:
        return self.getVirtualArgs()[2:4]

    def getX(self)->int:
        return self.getPos()[0]

    def getY(self)->int:
        return self.getPos()[1]

    def getWidth(self)->int:
        return self.getSize()[0]

    def getHeight(self)->int:
        return self.getSize()[1]
    # --------------------------------------

    # 根据下标索引来修改
    def updateIndexToArgs(self,i:int,value):
        args = list(self.getVirtualArgs())
        args[i] = value
        self.updateArgs(*args)

    # 移动
    def move(self,x,y):
        if self.__isRect():
            args = self.getVirtualArgs()[2:]
            self.updateArgs(x,y,*args)

    # 缩放
    def scale(self,proportion:float):
        if self.__isRect():
            x, y, w, h = self.getRect()
            args = self.getVirtualArgs()[4:]
            w,h = int(w*proportion),int(h*proportion)
            self.updateArgs(x,y,w,h,*args)

    # 检测鼠标是否点在图形上
    def isClick(self,pos:QPoint)->bool:
        if self.__isRect():
            x,y,w,h = self.getRect()
        else:
            x, y, w, h = self.getVirtualArgs()[:4]

        cx, cy = pos.x(), pos.y()
        if cx >= x and cx <= x + w and cy >= y and cy <= y + h:
            return True
        return False


# 虚拟图形对象管理类
class VirtualObjectManagement:
    def __init__(self):
        '''
            虚拟对象管理
        '''
        self.__virtualObjects = dict()  # type:typing.Dict[str:typing.List]

    def virtualObjName(self,vobj_name:str)->str:
        if self.isVirtualObject(vobj_name):
            return self.__virtualObjects[vobj_name]
        else:
            raise Exception("[{}] The virtual object does not exist!".format(vobj_name))

    def isVirtualObject(self,vobj_name:str)->bool:
        return True if self.__virtualObjects.get(vobj_name,None) else False

    def appendVirtualObject(self, vobj_name: str, **kwargs):
        if not self.isVirtualObject(vobj_name):
            self.__virtualObjects[vobj_name] = VirtualObject(vobj_name,**kwargs)


class SuperPainterAttr(QPainter):
    CurSor_Down = "down"
    CurSor_UP = "up"
    CurSor_Left = "left"
    CurSor_Right = "right"

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        self.__op = QPen(QColor(0,0,0))  # type:QPen
        self.__brush = QBrush(QColor(0,0,0)) # type: QBrush
        self.__font = None  # type: QFont
        self.__virtualObject = VirtualObjectManagement() # 虚拟对象

    def virtualObj(self,vobj_name:str)->VirtualObject:
        return self.__virtualObject.virtualObjName(vobj_name)

    def isVirtualObj(self,vobj_name:str)->bool:
        return self.__virtualObject.isVirtualObject(vobj_name)

    def setPen(self, op:QPen):
        if not isinstance(op,QPen):
            op = QPen(op)
        self.__op = op
        super().setPen(op)

    def setBrush(self, brush:QBrush):
        if not isinstance(brush,QBrush):
            brush = QBrush(brush)
        self.__brush = brush
        super().setBrush(brush)

    def open_(self) -> QPen:
        return self.__op

    def setFont(self, font:QFont):
        self.__font = font
        super().setFont(font)

    def isOpen(self) -> bool:
        return True if self.__op else False

    def isBrush(self) -> bool:
        return True if self.__brush else False

    def isFont(self) -> bool:
        return True if self.__font else False

    def clearOpen(self):
        if self.isOpen():
            self.setPen(qt.NoPen)

    def clearBrush(self):
        if self.isBrush():
            self.setBrush(qt.NoBrush)

    # 私有属性
    def __privateAttr(self, openAttr:dict=None,brushAttr:dict=None):
        if openAttr is None:
            openAttr = dict()

        if brushAttr is None:
            brushAttr = dict()

        op = QPen()
        open_color = openAttr.get("color") or openAttr.get("c")
        open_width = openAttr.get("width") or openAttr.get("w")
        open_style = openAttr.get("style")
        open_brush = openAttr.get("brush")
        open_capStyle = openAttr.get("cap") or openAttr.get("capStyle")
        open_cosmetic = openAttr.get("cos") or openAttr.get("cosmetic")
        open_joinStyle = openAttr.get("j_style") or openAttr.get("joinStyle")
        open_miterLimit = openAttr.get("miter") or openAttr.get("miterLimit")
        open_dashOffset = openAttr.get("dashOffset") or openAttr.get("off")
        open_dashPattern = openAttr.get("dashPattern") or openAttr.get("pattern")

        brush = QBrush(Qt.SolidPattern)
        brush_color = brushAttr.get("color") or brushAttr.get("c")
        brush_style = brushAttr.get("style")
        brush_matrix = brushAttr.get("matrix")
        brush_texture = brushAttr.get("texture") or brushAttr.get("tte")
        brush_transform = brushAttr.get("transform") or brushAttr.get("trans")
        brush_textureImage = brushAttr.get("textureImage") or brushAttr.get("image")

        if open_color:
            op.setColor(QColor(open_color))
        if open_width:
            op.setWidth(open_width)
        if open_style:
            op.setStyle(open_style)
        if open_brush:
            if isinstance(open_brush,str):
                open_brush = QColor(open_color)
            op.setBrush(open_brush)
        if open_capStyle:
            op.setCapStyle(open_capStyle)
        if open_cosmetic:
            op.setCosmetic(open_cosmetic)
        if open_joinStyle:
            op.setJoinStyle(open_joinStyle)
        if open_miterLimit:
            op.setMiterLimit(open_miterLimit)
        if open_dashOffset:
            op.setDashOffset(open_dashOffset)
        if open_dashPattern:
            op.setDashPattern(open_dashPattern)

        if brush_color:
            brush.setColor(QColor(brush_color))
        if brush_style:
            brush.setStyle(brush_style)
        if brush_matrix:
            brush.setMatrix(brush_matrix)
        if brush_texture:
            brush.setTexture(brush_texture)
        if brush_transform:
            brush.setTransform(brush_transform)
        if brush_textureImage:
            brush.setTextureImage(brush_textureImage)

        if openAttr:
            self.setPen(op)

        if brushAttr:
            self.setBrush(brush)

    def __restorePrivateAttr(self, op:QPen, brush:QBrush):
        if op:
            self.setPen(op)
        if brush:
            self.setBrush(brush)

    def decorator_to_all_draw_methods(self):
        def decorator(func):
            def wrapper(*args, **kwargs):
                op = self.open_()
                brush = self.brush()
                openAttr = kwargs.get("openAttr", None)
                brushAttr = kwargs.get("brushAttr", None)

                virtual_object_name = kwargs.get("virtualObjectName",None) or kwargs.get("vobj",None)
                if virtual_object_name:
                    self.__virtualObject.appendVirtualObject(virtual_object_name,func=func,args=args,kwargs=kwargs,
                                                             openAttr=openAttr,brushAttr=brushAttr)
                    vir_obj = self.virtualObj(virtual_object_name)
                    args = vir_obj.getVirtualArgs()
                    vir_openAttr = vir_obj.getVirtualOpenAttr()
                    vir_brushAttr = vir_obj.getVirtualBrushAttr()

                    if openAttr:openAttr = vir_openAttr
                    elif vir_openAttr:
                        openAttr = vir_openAttr
                        kwargs["openAttr"] = openAttr
                    if brushAttr:brushAttr = vir_brushAttr
                    elif vir_brushAttr:
                        brushAttr = vir_brushAttr
                        kwargs["brushAttr"] = brushAttr

                self.__privateAttr(openAttr, brushAttr)
                if openAttr: del kwargs["openAttr"]
                if brushAttr: del kwargs["brushAttr"]
                # if virtual_object_name: del kwargs["virtualObjectName"]

                if not kwargs.get("virtualObjectName"):
                    func(*args, **kwargs)
                elif virtual_object_name and vir_obj.isHide() is False:
                    func(*args, **kwargs)

                self.__restorePrivateAttr(op, brush)
            return wrapper

        self.drawRect = decorator(self.drawRect)
        self.drawRoundedRect = decorator(self.drawRoundedRect)
        self.drawLine = decorator(self.drawLine)
        self.drawLines = decorator(self.drawLines)
        self.drawArc = decorator(self.drawArc)
        self.drawEllipse = decorator(self.drawEllipse)
        self.drawPoint = decorator(self.drawPoint)
        self.drawPoints = decorator(self.drawPoints)
        self.drawText = decorator(self.drawText)
        self.drawStaticText = decorator(self.drawStaticText)
        self.drawPolygon = decorator(self.drawPolygon)
        self.drawPie = decorator(self.drawPie)
        self.drawPicture = decorator(self.drawPicture)
        self.drawChord = decorator(self.drawChord)
        self.drawRects = decorator(self.drawRects)
        self.drawPolyline = decorator(self.drawPolyline)
        self.drawPath = decorator(self.drawPath)
        self.drawConvexPolygon = decorator(self.drawConvexPolygon)
        self.drawGlyphRun = decorator(self.drawGlyphRun)
        self.drawPixmap = decorator(self.drawPixmap)
        self.drawImage = decorator(self.drawImage)
        self.drawPixmapFragments = decorator(self.drawPixmapFragments)
        self.drawTiledPixmap = decorator(self.drawTiledPixmap)
        # 自定义
        self.drawCursor = decorator(self.drawCursor)
        self.drawRhomboid = decorator(self.drawRhomboid)

    # ------------下面代码是作为工具的提示代码------------

    def drawRect(self, x:int,y:int,w:int,h:int,
                 openAttr:dict=None,brushAttr:dict=None,virtualObjectName:str="") -> None:
        super().drawRect(x,y,w,h)

    def drawRoundedRect(self, x:int,y:int,w:int,h:int,r:int=5,r2:int=5,
                        openAttr:dict=None,brushAttr:dict=None,virtualObjectName:str="") -> None:
        super().drawRoundedRect(x,y,w,h,r,r2)

    def drawLine(self, x:int,y:int,x1:int,y1:int,
                 openAttr:dict=None,brushAttr:dict=None,virtualObjectName:str="") -> None:
        super().drawLine(x,y,x1,y1)

    def drawLines(self,*args,openAttr:dict=None,brushAttr:dict=None,virtualObjectName:str="")->None:
        super().drawLines(*args)

    def drawPixmap(self,rect:QRect,pix):
        super().drawPixmap(rect,pix)

    # --------------------------自定义图形---------------
    # The method is called a "Cursor", but it functions more like a cursor
    def drawCursor(self,x:int,y:int,w:int,h:int,r:int=10,lw:int=30,direction="down",
                   openAttr:dict=None,brushAttr:dict=None,virtualObjectName:str=""):
        if direction == SuperPainter.CurSor_Left:
            self.drawLine(x-lw,y+h//2,x+w//2,y+h//2)
        elif direction == SuperPainter.CurSor_Right:
            self.drawLine(x+w//2, y + w // 2, x + lw, y + h // 2)
        elif direction == SuperPainter.CurSor_UP:
            self.drawLine(x + w//2, h-lw, x + w // 2, y + h // 2)
        else:
            self.drawLine(x + w//2, y + h // 2, x + w // 2, lw)
        self.drawRoundedRect(x,y,w,h,r,r)

    def drawRhomboid(self,x:int,y:int,ratio:float,openAttr:dict=None,brushAttr:dict=None,virtualObjectName:str=""):
        path = QPainterPath(QPoint(x,y))
        path.moveTo(x-ratio,y)

        path.lineTo(x,y-ratio)
        path.lineTo(x+ratio,y)
        path.lineTo(x,y+ratio)
        path.lineTo(x-ratio,y)
        self.fillPath(path,self.__brush)



class SuperPainter(SuperPainterAttr):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.decorator_to_all_draw_methods()




from PyQtGuiLib.header.Qt.qtWidgets import *
from PyQtGuiLib.header.Qt.qtGui import *
from PyQtGuiLib.header.Qt.qtCore import *
from PyQtGuiLib.header.Qt.qtSip import *
from PyQtGuiLib.header.Qt.qtUic import *

from PyQtGuiLib.header.Qt import qt
from PyQtGuiLib.header.py.common import *

from PyQtGuiLib.header.versions import PYQT_VERSIONS
from PyQtGuiLib.header.customStyle import CustomStyle
from PyQtGuiLib.header.utility import (
    is_mac_sys,
    is_win_sys,
    desktopCount,
    desktopSize,
    desktopAllSize,
    desktopCenter,
    textSize,
    rgbTohsv,
    loadUic,
    Widget
)
from PyQtGuiLib.header.error import *


# 处理mac下无法运行的情况
if is_mac_sys and PYQT_VERSIONS in ["PySide2","PySide6"]:
    os.environ["QT_MAC_WANTS_LAYER"] = "1"



# -*- coding:utf-8 -*-
# @time:2022/12/1018:13
# @author:LX
# @file:test_BubbleWidget.py
# @software:PyCharm

from PyQtGuiLib.header import PYQT_VERSIONS
from PyQtGuiLib.header import (
    sys,
    QApplication,
    QWidget,
    QPushButton,
    QVBoxLayout
)

from PyQtGuiLib.core import BubbleWidget

'''
    气泡窗口的测试用例
'''

class Test(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800,500)

        # self.setWindowOpacity(0.5)
        self.vboy = QVBoxLayout(self)

        self.btn = QPushButton("一号玩家",self)
        self.btn.resize(130,60)
        self.btn.move(400,200)

        self.btn2 = QPushButton("二号玩家", self)
        self.btn2.resize(130, 60)
        self.btn2.move(400, 400)

        self.vboy.addWidget(self.btn)
        self.vboy.addWidget(self.btn2)

        # 气泡控件
        self.bu_top = BubbleWidget(self)
        self.bu_top.setDirection(BubbleWidget.Down)
        self.bu_top.setText("hello world")
        self.bu_top.setTrack(self.btn)

        self.bu_top.setStyleSheet('''
BubbleWidget{
qproperty-backgroundColor: rgba(100, 130, 255,200);
qproperty-radius:10;
qproperty-fontSize:15;
qproperty-arrowsSize:12;
qproperty-margin:0;
}
''')


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = Test()
    win.show()

    if PYQT_VERSIONS in ["PyQt6", "PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2023/3/2914:11
# @author:LX
# @file:qssDrak.py
# @software:PyCharm

#
QSSDrak ='''

*{
background-color: rgb(50, 50, 50);
color:rgb(220, 220, 220);
font: 12pt "黑体";
}


QScrollBar:horizontal{
padding:0px;
max-height:12px;
padding:2px;
}
QScrollBar:vertical{
padding:0px;
max-width:12px;
padding:2px;
}
QScrollBar::handle{
background-color: rgb(39, 39, 39);
border-radius:4%;
}
QScrollBar::handle:hover{
background-color: rgb(72, 72, 72);
border:1px solid rgb(149, 149, 149);
}
QScrollBar::add-page,QScrollBar::sub-page {
background:rgb(57, 57, 57);
}
QScrollBar::sub-line,QScrollBar::add-line {
background:none;
}

QPushButton{
background-color: rgb(8, 8, 8);
border:2px solid rgb(230, 230, 230);
color:rgb(220, 220, 220);
border-radius:6%;
}
QPushButton:hover{
background-color: rgb(0, 0, 0);
}
QPushButton:pressed{
background-color:rgb(50, 50, 50);
}


QToolButton {
border-radius:6%;
background-color: #1E1E1E; 
color: #FFFFFF; 
border: 1px solid #2F2F2F; 
padding: 5px;
}
QToolButton:hover {
background-color: rgb(18, 18, 18);
}
QToolButton:pressed {
background-color:#1E1E1E; 
}


QCommandLinkButton{
background-color: rgb(8, 8, 8);
border:1px solid rgb(0, 120, 215);
color:rgb(220, 220, 220);
border-radius:6%;
}
QCommandLinkButton:hover{
background-color: rgb(0, 0, 0);
}
QCommandLinkButton:pressed{
background-color:rgb(50, 50, 50);
}


QListView {
outline: 0px;
background-color: rgb(2, 2, 2);
color: #FFFFFF;
border: 1px solid #666666;
}
QListView::item:hover{
padding:0px 0px 0px 15px;
color:rgb(255, 255, 255);
background-color: qlineargradient(spread:pad, x1:0, y1:0.5625, x2:0.994318, y2:0.614, stop:0 rgba(0, 0, 0, 255), stop:0.909091 rgba(73, 73, 73, 255));
}
QListView::item {
background-color: rgb(2, 2, 2);
color: rgb(143, 143, 143);
padding: 5px;
border-bottom: 1px solid #666666;
}
QListView::item:selected {
padding:0px 0px 0px 15px;
color:rgb(255, 255, 255);
background-color: qlineargradient(spread:pad, x1:0, y1:0.5625, x2:0.994318, y2:0.614, stop:0 rgba(0, 0, 0, 255), stop:0.909091 rgba(73, 73, 73, 255));
}



QTableView{
outline: 0px;
background-color:rgb(33, 33, 33);
gridline-color:rgb(125, 125, 125);
}
QTableView QHeaderView,QTableView QHeaderView::section, QTableView QTableCornerButton::section {
background-color:rgb(0, 0, 0);
color:rgb(175, 175, 175);
}
QTableView QHeaderView::section::pressed{
color:rgb(207, 207, 103);
}
QTableView QTableCornerButton::section{
background-color: rgb(0, 0, 0);
}
QHeaderView::section, QTableCornerButton:section {
padding: 3px;
margin: 0px;
border: 1px solid gray;
border-left-width: 0px;
border-right-width: 1px;
border-top-width: 0px;
border-bottom-width: 1px;
}
QTableView::indicator{
background-color: rgb(255, 170, 0);
}
QTableView::indicator:checked{
background-color:rgb(33, 33, 33);
}
QTableView::item{
padding:0px;
border:0px;
}
QTableView::item:selected{
background-color:#000;
}
QTableView::item:hover{
background-color: rgb(86, 86, 86);
}



QTreeView{
background-color: rgb(33, 33, 33);
color:rgb(188, 188, 188);
}
QTreeView QHeaderView,QTreeView QHeaderView::section{
background-color: rgb(0, 0, 0);
color:rgb(175, 175, 175);
}
QTreeView::item:hover{
background-color:transparent;
}
QTreeView::item:selected{
background-color:rgb(121, 121, 121);
color: rgb(255, 255, 255);
}
QTreeView::branch:selected{
background-color:rgb(121, 121, 121);
}


QGroupBox {
border:2px solid rgb(118, 118, 118);
border-radius:5px;
background-color: rgb(26, 26, 26);
}


QToolBox::tab {
background-color: rgb(90, 90, 90);
border-radius:3%;
padding:5px;
}
QToolBox::tab:selected{
font: italic;
color: white;
}



QTabWidget::pane {
}
QTabWidget QWidget{
background-color: rgb(20, 20, 20);
}
QTabWidget QTabBar::tab{
background-color: rgb(20, 20, 20);
min-width: 20ex;
padding: 5px;
}
QTabBar::tab:selected{
color: rgb(180, 239, 255);
}

QTabBar::tab:!selected{
background-color: rgb(65, 65, 65);
}



QComboBox {
background-color: rgb(0, 0, 0);
border:1px solid rgb(255, 255, 255);
border-bottom:none;
border-left:none;
border-right:none;
color: rgb(182, 182, 182);
border-radius: 1px;
padding: 1px 18px 1px 3px;
min-width: 6em;
}
QComboBox:editable{
}
QComboBox::drop-down{
}
QComboBox QAbstractItemView{
background-color: rgb(0, 0, 0);
border:none;
outline: 0px;
selection-background-color:rgb(0, 0, 0);
selection-color:rgb(0, 255, 255);
}


QFontComboBox{
border:2px solid rgb(103, 103, 103);
border-radius: 5px;
padding: 1px 18px 1px 3px;
min-width: 6em;
}
QFontComboBox QAbstractItemView{
outline: 0px;
selection-background-color: rgb(90, 90, 90);
}



QLineEdit{
border:2px solid rgb(0, 0, 0);
border-top:none;
border-left:none;
border-right:none;
}
QLineEdit:hover{
border:2px solid qlineargradient(spread:pad, x1:0.012, y1:0.556818, x2:1, y2:0.573864, stop:0 rgba(110, 203, 205, 255), stop:0.965909 rgba(67, 91, 95, 255));
border-top:none;
border-left:none;
border-right:none;
}



QTextEdit{
border:1px solid rgb(0, 0, 0);
border-radius: 3px;
}
QTextEdit:hover{
border:1px solid rgb(0, 85, 255);
}


QPlainTextEdit{
border:1px solid rgb(0, 0, 0);
border-radius: 4%;
background-color: rgb(66, 66, 66);
}
QPlainTextEdit:hover{
border:1px solid rgb(109, 109, 109);
}



QProgressBar {
border: 2px solid rgb(0, 0, 0);
border-radius: 5px;
}
QProgressBar::chunk {
border: 2px solid rgb(72, 72, 72);
width: 20px;
}
'''

QSSDrak2 ='''
/*=============overall============*/
*{
background-color: rgb(50, 50, 50);
color:rgb(220, 220, 220);
font: 12pt "黑体";
}


QScrollBar:horizontal{
padding:0px;
max-height:12px;
padding:2px;
}
QScrollBar:vertical{
padding:0px;
max-width:12px;
padding:2px;
}
QScrollBar::handle{
background-color: rgb(39, 39, 39);
border-radius:4%;
}
QScrollBar::handle:hover{
background-color: rgb(72, 72, 72);
border:1px solid rgb(149, 149, 149);
}
QScrollBar::add-page,QScrollBar::sub-page {
background:rgb(57, 57, 57);
}
QScrollBar::sub-line,QScrollBar::add-line {
background:none;
}
/* ============== Buttons =============== */
QPushButton{
background-color: rgb(8, 8, 8);
border:2px solid rgb(230, 230, 230);
color:rgb(220, 220, 220);
border-radius:6%;
}
QPushButton:hover{
background-color: rgb(0, 0, 0);
}
QPushButton:pressed{
background-color:rgb(50, 50, 50);
}

/* QToolButton */
QToolButton {
border-radius:6%;
background-color: #1E1E1E; 
color: #FFFFFF; 
border: 1px solid #2F2F2F; 
padding: 5px;
}
QToolButton:hover {
background-color: rgb(18, 18, 18);
}
QToolButton:pressed {
background-color:#1E1E1E; 
}

/* QCommandLinkButton */
QCommandLinkButton{
background-color: rgb(8, 8, 8);
border:1px solid rgb(0, 120, 215);
color:rgb(220, 220, 220);
border-radius:6%;
}
QCommandLinkButton:hover{
background-color: rgb(0, 0, 0);
}
QCommandLinkButton:pressed{
background-color:rgb(50, 50, 50);
}

/* ============ Item Views =============== */
QListView {
outline: 0px;
background-color: rgb(2, 2, 2);
color: #FFFFFF;
border: 1px solid #666666;
}
QListView::item:hover{
padding:0px 0px 0px 15px;
color:rgb(255, 255, 255);
background-color: qlineargradient(spread:pad, x1:0, y1:0.5625, x2:0.994318, y2:0.614, stop:0 rgba(0, 0, 0, 255), stop:0.909091 rgba(73, 73, 73, 255));
}
QListView::item {
background-color: rgb(2, 2, 2);
color: rgb(143, 143, 143);
padding: 5px;
border-bottom: 1px solid #666666;
}
QListView::item:selected {
padding:0px 0px 0px 15px;
color:rgb(255, 255, 255);
background-color: qlineargradient(spread:pad, x1:0, y1:0.5625, x2:0.994318, y2:0.614, stop:0 rgba(0, 0, 0, 255), stop:0.909091 rgba(73, 73, 73, 255));
}


/* QTableView */
QTableView{
outline: 0px;
background-color:rgb(33, 33, 33);
gridline-color:rgb(125, 125, 125);
}
QTableView QHeaderView,QTableView QHeaderView::section, QTableView QTableCornerButton::section {
background-color:rgb(0, 0, 0);
color:rgb(175, 175, 175);
}
QTableView QHeaderView::section::pressed{
color:rgb(207, 207, 103);
}
QTableView QTableCornerButton::section{
background-color: rgb(0, 0, 0);
}
QHeaderView::section, QTableCornerButton:section {
padding: 3px;
margin: 0px;
border: 1px solid gray;
border-left-width: 0px;
border-right-width: 1px;
border-top-width: 0px;
border-bottom-width: 1px;
}
QTableView::indicator{
background-color: rgb(255, 170, 0);
}
QTableView::indicator:checked{
background-color:rgb(33, 33, 33);
}
QTableView::item{
padding:0px;
border:0px;
}
QTableView::item:selected{
background-color:#000;
}
QTableView::item:hover{
background-color: rgb(86, 86, 86);
}


/* QTreeView */
QTreeView{
background-color: rgb(33, 33, 33);
color:rgb(188, 188, 188);
}
QTreeView QHeaderView,QTreeView QHeaderView::section{
background-color: rgb(0, 0, 0);
color:rgb(175, 175, 175);
}
QTreeView::item:hover{
background-color:transparent;
}
QTreeView::item:selected{
background-color:rgb(121, 121, 121);
color: rgb(255, 255, 255);
}
QTreeView::branch:selected{
background-color:rgb(121, 121, 121);
}

/* ================= Containers ================= */
QGroupBox {
border:2px solid rgb(118, 118, 118);
border-radius:5px;
background-color: rgb(26, 26, 26);
}

/* QToolBox */
QToolBox::tab {
background-color: rgb(90, 90, 90);
border-radius:3%;
padding:5px;
}
QToolBox::tab:selected{
font: italic;
color: white;
}


/*  QTabWidget  */
QTabWidget::pane {
}
QTabWidget QWidget{
background-color: rgb(20, 20, 20);
}
QTabWidget QTabBar::tab{
background-color: rgb(20, 20, 20);
min-width: 20ex;
padding: 5px;
}
QTabBar::tab:selected{
color: rgb(180, 239, 255);
}

QTabBar::tab:!selected{
background-color: rgb(65, 65, 65);
}


/* =================Input Widgets ==============*/
QComboBox {
background-color: rgb(0, 0, 0);
border:1px solid rgb(255, 255, 255);
border-bottom:none;
border-left:none;
border-right:none;
color: rgb(182, 182, 182);
border-radius: 1px;
padding: 1px 18px 1px 3px;
min-width: 6em;
}
QComboBox:editable{
}
QComboBox::drop-down{
}
QComboBox QAbstractItemView{
background-color: rgb(0, 0, 0);
border:none;
outline: 0px;
selection-background-color:rgb(0, 0, 0);
selection-color:rgb(0, 255, 255);
}

/* QFontComboBox */
QFontComboBox{
border:2px solid rgb(103, 103, 103);
border-radius: 5px;
padding: 1px 18px 1px 3px;
min-width: 6em;
}
QFontComboBox QAbstractItemView{
outline: 0px;
selection-background-color: rgb(90, 90, 90);
}


/* QLineEdit */
QLineEdit{
border:2px solid rgb(0, 0, 0);
border-top:none;
border-left:none;
border-right:none;
}
QLineEdit:hover{
border:2px solid qlineargradient(spread:pad, x1:0.012, y1:0.556818, x2:1, y2:0.573864, stop:0 rgba(110, 203, 205, 255), stop:0.965909 rgba(67, 91, 95, 255));
border-top:none;
border-left:none;
border-right:none;
}


/* QTextEdit */
QTextEdit{
border:1px solid rgb(0, 0, 0);
border-radius: 3px;
}
QTextEdit:hover{
border:1px solid rgb(0, 85, 255);
}

/* QPlainTextEdit */
QPlainTextEdit{
border:1px solid rgb(0, 0, 0);
border-radius: 4%;
background-color: rgb(66, 66, 66);
}
QPlainTextEdit:hover{
border:1px solid rgb(109, 109, 109);
}


/* QProgressBar */
QProgressBar {
border: 2px solid rgb(0, 0, 0);
border-radius: 5px;
}
QProgressBar::chunk {
border: 2px solid rgb(72, 72, 72);
width: 20px;
}
'''


# -*- coding:utf-8 -*-
# @time:2023/5/1217:39
# @author:LX
# @file:libStyle.py
# @software:PyCharm

from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QWidget,
    QStyle,
    QStyleOption,
    QPainter,
    QStyleHintReturn,
    QStyleOptionButton,
    QPalette,
    QColor,
    QRect,
    QStylep
)

class MyStyle(QStyle):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

    def styleHint(self, stylehint: QStyle.StyleHint, option: QStyleOption, widget: QWidget, returnData: QStyleHintReturn) -> int:
        return 50

    def drawPrimitive(self, pe: QStyle.PrimitiveElement, opt: QStyleOption, p: QPainter, widget: QWidget) -> None:
        if pe == QStyle.SP_TitleBarMaxButton:
            print("Das")

        super().drawPrimitive(pe,opt,p,widget)


class Test(QWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.resize(500,300)

    def paintEvent(self, event):
        painter = QPainter(self)
        opt = QStyleOptionButton()
        opt.initFrom(self)

        self.style().drawPrimitive(QStyle.PE_Widget, opt, painter, self)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    app.setStyle(MyStyle())

    win = Test()
    win.show()

    if PYQT_VERSIONS in ["PyQt6","PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())



# -*- coding:utf-8 -*-
# @time:2023/2/209:09
# @author:LX
# @file:component.py
# @software:PyCharm
'''
    小组件  -- 这个文件不能单独运行
'''
from PyQtGuiLib.header import (
    QPoint,
    QGroupBox,
    QFormLayout,
    QPushButton,
    QSize,
    QColor,
    QLabel,
    QGridLayout,
    QSpinBox,
    QDoubleSpinBox,
    QComboBox,
    QFileDialog,
    QFontComboBox,
    QMessageBox,
    QGraphicsDropShadowEffect,
    QGraphicsBlurEffect,
    QPropertyAnimation
)
from PyQtGuiLib.core import PaletteFrame
from PyQtGuiLib.core.switchButtons import SwitchButton
from PyQtGuiLib.styles import QssStyleAnalysis

# 通用 QGroupBox 大小(最多3排组件)
PUBLIC_GROUPBOX_SIZE = QSize(160,100)
PUBLIC_GROUPBOX_SIZE_4 = QSize(160,130)


class GroupBoxABC:
    def __init__(self,styleLinker,parent):
        self.__title = ""
        self.__styleLinker = styleLinker
        self.__parent = parent

    def setTitle(self,title:str):
        self.__title = title

    def title(self)->str:
        return self.__title

    def styleLinker(self):
        return self.__styleLinker

    def parent(self):
        return self.__parent

    # 被操作的控件对象
    def controlObj(self):
        return self.styleLinker().global_var.parent()

    # 通用打开调色版
    def openPaletteFrame(self, title, callfun, argc=None):
        p = PaletteFrame()  # 创建颜色版
        p.setWindowTitle(title)
        p.show()

        if argc:
            p.rgbaChange.connect(lambda rgba: callfun(rgba, argc))
        else:
            p.rgbaChange.connect(lambda rgba: callfun(rgba))

    # 返回当前操作的选择器
    def selector(self):
        if self.styleLinker().global_select is None:
            head = self.styleLinker().global_var.header()[0]
        else:
            head = self.styleLinker().global_select
        return self.styleLinker().global_var.selector(head)

    # 通用修改属性的方法
    def updateAttr_(self, key, value):
        '''
        :param key: 样式属性
        :param value: 样式值
        :return:
        '''
        self.selector().updateAttr(key, value)
        self.updateStyleCode()

    # 检查属性
    def isAttr_(self,key)->bool:
        return self.selector().isAttr(key)

    # 移除属性
    def removerAttr_(self,key):
        if self.isAttr_(key):
            self.selector().removeAttr(key)
            self.updateStyleCode()

    # 更新样式代码
    def updateStyleCode(self)->None:
        self.styleLinker().showStyleBrowserCode(self.styleLinker().global_var.toStr())

    # 创建通用 外框
    def getGroupBox_(self, size=None):
        groupBox = QGroupBox()
        self.parent().addGroupBox(groupBox)
        groupBox.setTitle(self.title())
        if size:
            groupBox.setFixedSize(size)
        else:
            groupBox.setFixedSize(PUBLIC_GROUPBOX_SIZE)
        return groupBox

    # 子类实现具体的功能
    def module(self):
        pass


# 通用的颜色组件
class ColorComponent(GroupBoxABC):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.setTitle("调色区")

        #  图片设置的标记
        self.isImage = False

    def open_PaletteFrame(self,btn, case: str):
        if case in ["bg", "c"]:
            p = PaletteFrame()  # 创建颜色版
            p.show()

        def update(rgba, k, v):
            self.updateAttr_(k, v)
            btn.setStyleSheet('''background-color:rgba(%s, %s, %s,%s);''' % rgba)

        if case == "bg":
            p.setWindowTitle("背景色")
            p.rgbaChange.connect(lambda rgba: update(rgba, "background-color", "rgba(%s, %s, %s,%s)" % rgba))

        if case == "c":
            p.setWindowTitle("前景色")
            p.rgbaChange.connect(lambda rgba: update(rgba, "color", "rgba(%s, %s, %s,%s)" % rgba))

        if case == "image":
            if self.isImage:
                reply = QMessageBox.critical(btn,"警告","确认删除背景图片",QMessageBox.Yes|QMessageBox.No)
                if reply == QMessageBox.Yes:
                    self.removerAttr_("border-image")
                    self.isImage = False
                    btn.setText("...")
                    btn.setStyleSheet("")
            else:
                name, ty = QFileDialog.getOpenFileName(self.parent(), "选择图片", "", "images(*.png *.jpg *.jpeg *.bmp *.gif)")
                if name:
                    self.updateAttr_("border-image", "url(%s)" % name)
                    btn.setStyleSheet('''
QPushButton{
border:2px solid #fff;
background-color: rgb(255, 0, 0);
color: rgb(255, 255, 255);
}
                    ''')
                    btn.setText("移除图片")
                    self.isImage = True

    def module(self):
        groupBox = self.getGroupBox_()
        fboy = QFormLayout(groupBox)
        fboy.setContentsMargins(3, 3, 3, 3)

        '''
            这里需要重新编写结构,增加一个清除图片
        '''

        bgc = QLabel("背景颜色")
        bgc_btn = QPushButton()
        color = QLabel("前景色")
        color_btn = QPushButton()
        image_l = QLabel("背景图片(b)")
        image_btn = QPushButton("...")
        fboy.addRow(bgc, bgc_btn)
        fboy.addRow(color, color_btn)
        fboy.addRow(image_l, image_btn)

        # event
        bgc_btn.clicked.connect(lambda: self.open_PaletteFrame(bgc_btn, "bg"))
        color_btn.clicked.connect(lambda: self.open_PaletteFrame(color_btn, "c"))
        image_btn.clicked.connect(lambda: self.open_PaletteFrame(image_btn, "image"))


# 通用调节大小位置组件
class GeometryComponent(GroupBoxABC):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.setTitle("位置大小")

    def geometry_event(self,case, v):
        parent = self.styleLinker().global_var.parent()
        if case == "x":
            parent.move(v, parent.y())

        if case == "y":
            parent.move(parent.x(), v)

        if case == "w":
            parent.resize(v, parent.height())

        if case == "h":
            parent.resize(parent.width(), v)

    def module(self):
        groupBox = self.getGroupBox_()
        gboy = QGridLayout(groupBox)
        gboy.setContentsMargins(3, 3, 3, 3)

        xy_l = QLabel("位置")
        x_spinbox = QSpinBox()
        y_spinbox = QSpinBox()
        x_spinbox.setMaximum(600)
        y_spinbox.setMaximum(600)
        x_spinbox.setValue(self.styleLinker().global_var.parent().x())
        y_spinbox.setValue(self.styleLinker().global_var.parent().y())

        gboy.addWidget(xy_l, 0, 0)
        gboy.addWidget(x_spinbox, 0, 1)
        gboy.addWidget(y_spinbox, 0, 2)

        x_spinbox.valueChanged.connect(lambda v: self.geometry_event("x", v))
        y_spinbox.valueChanged.connect(lambda v: self.geometry_event("y", v))

        size_l = QLabel("大小")
        w_spinbox = QSpinBox()
        h_spinbox = QSpinBox()
        w_spinbox.setMaximum(600)
        h_spinbox.setMaximum(600)
        w_spinbox.setValue(self.styleLinker().global_var.parent().width())
        h_spinbox.setValue(self.styleLinker().global_var.parent().height())

        gboy.addWidget(size_l, 1, 0)
        gboy.addWidget(w_spinbox, 1, 1)
        gboy.addWidget(h_spinbox, 1, 2)

        w_spinbox.valueChanged.connect(lambda v: self.geometry_event("w", v))
        h_spinbox.valueChanged.connect(lambda v: self.geometry_event("h", v))


# 通用边组件
class BorderComponent(GroupBoxABC):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.setTitle("边设置")

    def border_event(self,case,v,color_btn=None):
        if case == "border_radius":
            self.updateAttr_("border-radius","{}px".format(v))
        if case == "border_width":
            self.updateAttr_( "border-width", "{}px".format(v))
        if case == "border_style":
            self.updateAttr_("border-style", "%s"%v)
        if case == "border_color":
            p = PaletteFrame()  # 创建颜色版
            p.show()

            def update(rgba, k, v):
                self.updateAttr_(k, v)
                color_btn.setStyleSheet('''background-color:rgba(%s, %s, %s,%s);''' % rgba)

            p.rgbaChange.connect(lambda rgba: update(rgba, "border-color", "rgba(%s, %s, %s,%s)" % rgba))

    def module(self):
        groupBox = self.getGroupBox_(PUBLIC_GROUPBOX_SIZE_4)
        gboy = QGridLayout(groupBox)
        gboy.setContentsMargins(3, 3, 3, 3)

        radius_l = QLabel("圆角")
        radius_spinbox = QSpinBox()
        width_l = QLabel("边框宽度")
        width_spinbox = QSpinBox()
        style_l = QLabel("边框风格")
        style_combox = QComboBox()
        style_combox.addItems(["none", "solid", "outset", "inset",
                               "dashed", "dot-dash", "dot-dot-dash",
                               "dotted", "double", "groove", "ridge"])
        color_l = QLabel("边框颜色")
        color_btn = QPushButton()

        gboy.addWidget(radius_l, 0, 0)
        gboy.addWidget(radius_spinbox, 0, 1)
        gboy.addWidget(width_l, 1, 0)
        gboy.addWidget(width_spinbox, 1, 1)
        gboy.addWidget(style_l, 2, 0)
        gboy.addWidget(style_combox, 2, 1)
        gboy.addWidget(color_l, 3, 0)
        gboy.addWidget(color_btn, 3, 1)

        radius_spinbox.valueChanged.connect(lambda v: self.border_event("border_radius", v))
        width_spinbox.valueChanged.connect(lambda v: self.border_event("border_width", v))
        style_combox.currentTextChanged.connect(lambda v: self.border_event("border_style", v))
        style_combox.textHighlighted.connect(lambda v: self.border_event("border_style", v))
        color_btn.clicked.connect(lambda: self.border_event("border_color", 0, color_btn))


# 通用字体类组件
class FontComponent(GroupBoxABC):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.setTitle("文字操作")

    def font_event(self,case,v):
        if case == "font_size":
            self.updateAttr_("font-size","{}px".format(v))
        if case == "text_decoration":
            self.updateAttr_("text-decoration", "%s"%v)
        if case == "font_family":
            self.updateAttr_("font-family", "\'%s\'" % v)
        if case == "font_style":
            self.updateAttr_("font-style", "%s" % v)

    def module(self):
        groupBox = self.getGroupBox_(PUBLIC_GROUPBOX_SIZE_4)
        gboy = QGridLayout(groupBox)
        gboy.setContentsMargins(3, 3, 3, 3)

        size_l = QLabel("文字大小")
        size_spinbox = QSpinBox()
        size_spinbox.setValue(12)
        decoration_l = QLabel("文字装饰")
        decoration_combobox = QComboBox()
        decoration_combobox.addItems(["none", "underline", "line-through"])
        family_l = QLabel("字体")
        family_fcombobox = QFontComboBox()
        style_l = QLabel("文字风格")
        style_combobox = QComboBox()
        style_combobox.addItems(["none", "italic", "oblique"])

        gboy.addWidget(size_l, 0, 0)
        gboy.addWidget(size_spinbox, 0, 1)
        gboy.addWidget(decoration_l, 1, 0)
        gboy.addWidget(decoration_combobox, 1, 1)
        gboy.addWidget(family_l, 2, 0)
        gboy.addWidget(family_fcombobox, 2, 1)
        gboy.addWidget(style_l, 3, 0)
        gboy.addWidget(style_combobox, 3, 1)

        size_spinbox.valueChanged.connect(lambda v: self.font_event("font_size", v))
        decoration_combobox.currentTextChanged.connect(lambda v: self.font_event("text_decoration", v))
        decoration_combobox.textHighlighted.connect(lambda v: self.font_event("text_decoration", v))
        family_fcombobox.currentTextChanged.connect(lambda v: self.font_event("font_family", v))
        family_fcombobox.textHighlighted.connect(lambda v: self.font_event("font_family", v))
        style_combobox.currentTextChanged.connect(lambda v: self.font_event("font_style", v))
        style_combobox.textHighlighted.connect(lambda v: self.font_event("font_style", v))

# ---------------------------------------------------


# 通用边细节组件(手动加载)
class BorderDetailComponent(GroupBoxABC):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.setTitle("边细节")
        self._dir = None # 方向设置

    def setDir(self,dir,w_spinbox:QSpinBox,c_btn:QPushButton,s_combobox:QComboBox):
        if dir == "none":
            '''
                如果当 border-{}-width:0px;,那么这句话前面的 border-width:10px,
                将会永久失效,所有在选择方向为 none时,删除当前 border-{}-width 这个属性,来保障,
                border-width是有效的,并重置其他控件
            '''
            temp_dir = ["top","right","bottom","left"]
            temp_attr = ["width","color","style"]
            for d in temp_dir:
                for a in temp_attr:
                    key = "border-{}-{}".format(d,a)
                    self.removerAttr_(key)

            # 这里顺序不能反,必须先重置方向,防止触发事件
            self._dir = None
            w_spinbox.setValue(0)
            c_btn.setStyleSheet("")
            s_combobox.setCurrentIndex(0)
            return
        self._dir = dir

    def isDir(self) -> bool:
        return bool(self._dir)

    def border_event(self,case,v,btn):
        if self.isDir() and case == "b_width":
            attr = "border-{}-width".format(self._dir)
            self.updateAttr_(attr,"%s"%v)

        if self.isDir() and case == "b_color":
            attr = "border-{}-color".format(self._dir)
            def call(rgba):
                btn.setStyleSheet("background-color:rgba(%s, %s, %s,%s);"%rgba)
                self.updateAttr_(attr,"rgba(%s, %s, %s,%s)"%rgba)
            self.openPaletteFrame("[{}]颜色".format(self._dir),call)

        if self.isDir() and case == "b_style":
            attr = "border-{}-style".format(self._dir)
            self.updateAttr_(attr,"%s"%v)

    def module(self):
        groupBox = self.getGroupBox_(PUBLIC_GROUPBOX_SIZE_4)
        fboy = QFormLayout(groupBox)
        fboy.setContentsMargins(3, 3, 3, 3)

        direction_l = QLabel("方向")
        direction_combobox = QComboBox()
        direction_combobox.addItems(["none","top","right","bottom","left"])
        width_l = QLabel("边宽")
        width_spinbox = QSpinBox()
        color_l = QLabel("边颜色")
        color_btn = QPushButton()
        style_l = QLabel("边风格")
        style_combobox = QComboBox()
        style_combobox.addItems(["none", "solid", "outset", "inset",
                               "dashed", "dot-dash", "dot-dot-dash",
                               "dotted", "double", "groove", "ridge"])

        fboy.addRow(direction_l, direction_combobox)
        fboy.addRow(width_l, width_spinbox)
        fboy.addRow(color_l, color_btn)
        fboy.addRow(style_l, style_combobox)

        direction_combobox.currentTextChanged.connect(lambda text:self.setDir(text,width_spinbox,color_btn,style_combobox))
        width_spinbox.valueChanged.connect(lambda v:self.border_event("b_width",v,None))
        color_btn.clicked.connect(lambda :self.border_event("b_color",None,color_btn))
        style_combobox.currentTextChanged.connect(lambda v:self.border_event("b_style",v,None))
        style_combobox.textHighlighted.connect(lambda v:self.border_event("b_style",v,None))


# 通用边细节组件-圆角(手动加载)
class BorderDetailRadiusComponent(GroupBoxABC):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.setTitle("边细节-圆角")
        self._dir = None # 方向设置

    def isDir(self)->bool:
        return bool(self._dir)

    def setDir(self,dir):
        if dir == "none":
            attr = "border-{}-radius".format(self._dir)
            self.removerAttr_(attr)
            return
        self._dir = dir

    def radius_event(self,v):
        attr = "border-{}-radius".format(self._dir)
        if self.isDir():
            self.updateAttr_(attr,"%s"%v)

    def module(self):
        groupBox = self.getGroupBox_(PUBLIC_GROUPBOX_SIZE_4)
        fboy = QFormLayout(groupBox)
        fboy.setContentsMargins(3, 3, 3, 3)

        direction_l = QLabel("方向")
        direction_combobox = QComboBox()
        direction_combobox.addItems(["none", "top-left", "top-right", "bottom-left", "bottom-right"])
        radius_l = QLabel("圆角大小")
        radius_spinbox = QSpinBox()

        fboy.addRow(direction_l, direction_combobox)
        fboy.addRow(radius_l, radius_spinbox)

        direction_combobox.currentTextChanged.connect(self.setDir)
        radius_spinbox.valueChanged.connect(self.radius_event)


# 通用阴影组件(手动加载)
class ShadowComponent(GroupBoxABC):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.setTitle("阴影(非QSS)")

        self._shawObj = QGraphicsDropShadowEffect()
        self.offx,self.offy =0,0
        self.r = 5
        self.s_color = QColor(0,255,0)

        self._shawObj.setOffset(self.offx,self.offy)
        self._shawObj.setBlurRadius(self.r)
        self._shawObj.setColor(self.s_color)

        # 阴影开关
        self.isShadow = False

    def shadow_event(self,case,v,btn):
        if case == "click":
            if self.isShadow is False:
                self.isShadow = True
                self.controlObj().setGraphicsEffect(self._shawObj)
            else:
                self.isShadow = False
                self.controlObj().setGraphicsEffect(None)
                '''
                    这里情况会和BlurComponent一致
                '''
                self._shawObj = QGraphicsDropShadowEffect()
                self._shawObj.setOffset(self.offx, self.offy)
                self._shawObj.setBlurRadius(self.r)
                self._shawObj.setColor(self.s_color)
            # self._shawObj.setEnabled(self.isShadow)
            # self._shawObj.clearGraphicsEffect()

            if self.isShadow is False:
                return

        if self.isShadow and case == "offX":
            self.offx = v
        if self.isShadow and case == "offY":
            self.offy = v
        if self.isShadow and case == "r":
            self.r = v
        if self.isShadow and case == "color":
            def color_(rgba):
                r,g,b,a = rgba
                self.s_color = QColor(int(r),int(g),int(b),int(a))
                self._shawObj.setColor(self.s_color)
                btn.setStyleSheet('''background-color:rgba(%s, %s, %s,%s);''' % rgba)
            self.openPaletteFrame("阴影颜色",color_)

        self._shawObj.setOffset(self.offx,self.offy)
        self._shawObj.setBlurRadius(self.r)
        # self._shawObj.setColor(self.s_color)

        # 控件对象
        self.controlObj().setGraphicsEffect(self._shawObj)

    def module(self):
        groupBox = self.getGroupBox_(PUBLIC_GROUPBOX_SIZE_4)
        gboy = QGridLayout(groupBox)
        gboy.setContentsMargins(3, 3, 3, 3)

        shaow_l = QLabel("阴影开关")
        shadow_btn = SwitchButton()
        off_l = QLabel("偏移")
        off_x = QSpinBox()
        off_y = QSpinBox()
        blur_radius_l = QLabel("阴影半径")
        blur_radius_spinbox = QSpinBox()
        blur_radius_spinbox.setValue(self.r)
        color_l = QLabel("阴影颜色")
        color_btn = QPushButton()


        gboy.addWidget(shaow_l,0,0)
        gboy.addWidget(shadow_btn,0,1,1,2)
        gboy.addWidget(off_l,1,0)
        gboy.addWidget(off_x,1,1)
        gboy.addWidget(off_y,1,2)
        gboy.addWidget(blur_radius_l,2,0)
        gboy.addWidget(blur_radius_spinbox,2,1,1,2)
        gboy.addWidget(color_l,3,0)
        gboy.addWidget(color_btn,3,1,1,2)

        shadow_btn.clicked.connect(lambda :self.shadow_event("click",None,None))
        off_x.valueChanged.connect(lambda v:self.shadow_event("offX",v,None))
        off_y.valueChanged.connect(lambda v:self.shadow_event("offY",v,None))
        blur_radius_spinbox.valueChanged.connect(lambda v:self.shadow_event("r",v,None))
        color_btn.clicked.connect(lambda v:self.shadow_event("color",v,color_btn))


# 通用模糊组件(手动加载)
class BlurComponent(GroupBoxABC):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.setTitle("模糊(非QSS)")

        self.hints = {
            "PerformanceHint":QGraphicsBlurEffect.PerformanceHint, # 注重模糊性能
            "QualityHint":QGraphicsBlurEffect.QualityHint, # 注重模糊质量
            "AnimationHint":QGraphicsBlurEffect.AnimationHint # 用于动画
        }

        self._blurObj = QGraphicsBlurEffect()
        self.blur_r = 5
        self._blurObj.setBlurRadius(self.blur_r)

        # 阴影开关
        self.isBlur = False

    def blur_event(self,case,v):
        if case == "click":
            if self.isBlur is False:
                self.isBlur = True
                self.controlObj().setGraphicsEffect(self._blurObj)
            else:
                self.isBlur = False
                self.controlObj().setGraphicsEffect(None)
                '''
                    self.controlObj().setGraphicsEffect(None)
                    这句执行后,会导致QGraphicsBlurEffect被删除,那么在下面
                    重新创建一次
                '''
                self._blurObj = QGraphicsBlurEffect()
                self._blurObj.setBlurRadius(self.blur_r)
            if self.isBlur is False:
                return

        if self.isBlur and case == "r":
            self.blur_r = v

        self._blurObj.setBlurRadius(self.blur_r)

        # 控件对象
        self.controlObj().setGraphicsEffect(self._blurObj)

    def module(self):
        groupBox = self.getGroupBox_(PUBLIC_GROUPBOX_SIZE)
        gboy = QGridLayout(groupBox)
        gboy.setContentsMargins(3, 3, 3, 3)

        shaow_l = QLabel("模糊开关")
        shadow_btn = SwitchButton()
        blur_radius_l = QLabel("模糊半径")
        blur_radius_spinbox = QDoubleSpinBox()
        blur_radius_spinbox.setValue(self.blur_r)
        hints_l = QLabel("模糊质量")
        hints_combobox = QComboBox()
        hints_combobox.addItems(list(self.hints.keys()))

        gboy.addWidget(shaow_l,0,0)
        gboy.addWidget(shadow_btn,0,1,1,2)
        gboy.addWidget(blur_radius_l,2,0)
        gboy.addWidget(blur_radius_spinbox,2,1,1,2)
        gboy.addWidget(hints_l,3,0)
        gboy.addWidget(hints_combobox,3,1,1,2)

        shadow_btn.clicked.connect(lambda :self.blur_event("click",None))
        blur_radius_spinbox.valueChanged.connect(lambda v:self.blur_event("r",v))


# 通用移动动画组件
class AniComponent(GroupBoxABC):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.setTitle("移动动画(非QSS)")

        print(self.controlObj(),self.controlObj().parent())

        self.moveAni = QPropertyAnimation()
        self.moveAni.setTargetObject(self.controlObj())
        self.moveAni.setPropertyName(b"pos")

        # 控件原始位置
        self.original_pos = self.controlObj().pos()

        # 动画事件
        self.msecs = 2

        self.end = QPoint(0,0)

        self.isAni = True

    def ani_finish_event(self,btn):
        btn.setDefaultState(False)
        self.isAni = True
        self.controlObj().move(self.original_pos)


    def start_event(self):
        if self.isAni:
            self.isAni = False
            self.moveAni.start()


    def ani_event(self,case:str,v):

        if self.isAni and case == "time":
            self.msecs = v
            self.moveAni.setDuration(self.msecs*1000)
        if self.isAni and case == "ani_x":
            self.end.setX(v)
        if self.isAni and case == "ani_y":
            self.end.setY(v)

        if self.isAni:
            self.moveAni.setStartValue(self.controlObj().pos())
            self.moveAni.setEndValue(self.end)


    def module(self):
        groupBox = self.getGroupBox_(PUBLIC_GROUPBOX_SIZE_4)
        gboy = QGridLayout(groupBox)
        gboy.setContentsMargins(1, 1, 1, 1)

        # --
        switch_l = QLabel("动画开关")
        switch_btn = SwitchButton()
        time_l = QLabel("动画时长")
        time_spinbox = QSpinBox()
        time_spinbox.setValue(self.msecs)
        pos_s_l = QLabel("起始位置")
        pos_s_combox = QComboBox()
        pos_s_combox.addItem("pos")
        pos_e_l = QLabel("结束位置")
        pos_e_spinbox_x = QSpinBox()
        pos_e_spinbox_y = QSpinBox()
        pos_e_spinbox_x.setMaximum(2000)
        pos_e_spinbox_y.setMaximum(2000)

        gboy.addWidget(switch_l,0,0,1,2)
        gboy.addWidget(switch_btn,0,1,1,2)
        gboy.addWidget(time_l,1,0,1,2)
        gboy.addWidget(time_spinbox,1,1,1,2)
        gboy.addWidget(pos_s_l,2,0,1,2)
        gboy.addWidget(pos_s_combox,2,1,1,2)
        gboy.addWidget(pos_e_l,3,0)
        gboy.addWidget(pos_e_spinbox_x,3,1)
        gboy.addWidget(pos_e_spinbox_y,3,2)

        switch_btn.clicked.connect(self.start_event)
        time_spinbox.valueChanged.connect(lambda v:self.ani_event("time",v))
        pos_e_spinbox_x.valueChanged.connect(lambda v:self.ani_event("ani_x",v))
        pos_e_spinbox_y.valueChanged.connect(lambda v:self.ani_event("ani_y",v))

        self.moveAni.finished.connect(lambda :self.ani_finish_event(switch_btn))




# -----------------------------------


# 小控件注册器
class RegisterComponent:
    '''
        __Reg_Funs: 是由程序自动加载的组件
        __Hand_Reg_Funs: 是由用户手动的使用右键来添加的组件

    '''
    __Reg_Funs =[
        ColorComponent,
        GeometryComponent,
        BorderComponent,
        FontComponent
    ]

    __Hand_Reg_Funs = [
        BorderDetailComponent,
        BorderDetailRadiusComponent,
        ShadowComponent,
        BlurComponent,
        AniComponent
    ]


    # 返回注册项
    @staticmethod
    def getRegister():
        return RegisterComponent.__Reg_Funs

    # 返回手动注册项
    @staticmethod
    def getHandRegister():
        return RegisterComponent.__Hand_Reg_Funs


# -*- coding:utf-8 -*-
# @time:2023/2/918:26
# @author:LX
# @file:eg2.py


from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    qt,
    QPushButton,
    QLabel,
    Qt,
    QFrame,
    QStyleOption,
    QPainter,
    QStyle
)

from PyQtGuiLib.styles import QssStyleAnalysis


class Test(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(600,600)


        self.setAttribute(Qt.WA_StyledBackground,True)

        self.l = QLabel("测试标签",self)
        self.btn = QPushButton("测试按钮",self)
        self.btn.setObjectName("btn")
        self.btn2 = QPushButton("测试按钮2号",self)
        self.l.move(30, 30)
        self.btn.resize(150, 80)
        self.btn.move(80, 80)
        self.btn2.resize(100,60)
        self.btn2.move(250,80)

        '''
            解析 测试2
        '''

        # 创建一个针对整个窗口的 QSS 解析器
        self.qss = QssStyleAnalysis(self)
        # 对窗口上所有按钮,标签设置样式
        self.qss.setQSS('''
        QPushButton{
        color: rgb(0, 255, 127);
        background-color:rgb(0, 170, 0);
        }
        QLabel{
        color: rgb(42, 55, 127);
        background-color:rgb(255, 170, 0);
        }
        ''')

        print("-------------------------------")
        print(self.qss.selector("QPushButton").attr("color")) # 获取该选择器下面某个属性的值
        self.qss.selector("QPushButton").updateAttr("background-color","red") # 修改背景样式
        # self.qss.selector("QLabel").removeAttr("background-color") # 移除该标签的属性
        self.qss.removeSelector("QPushButton") # 移除该选择器的所有样式

        print("--------")
        self.qss.appendQSSDict({
            "QPushButton":{
                "border":"2px solid red"
            }
        })
        self.qss.appendQSS('''
        QPushButton:hover{
        border:2px solid blue;
        }
        ''')


if __name__ == '__main__':
    app = QApplication(sys.argv)

    win = Test()
    win.show()

    if PYQT_VERSIONS in ["PyQt6","PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding: utf-8 -*-

class PyStyle:
    """样式集合"""

    windowQFStyle = """
        #windowQF {{
            background-color: {_bg_color};
            network-radius: {_border_radius};
            network: {_border_size}px solid {_border_color};
        }}
        QFrame {{ 
            color: {_text_color};
            font: {_text_font};
        }}
    """
    TEBgQF = """
        #TEBgQF {{
            background-color: {_bg_color};
            network-radius: {_border_radius};
            network: {_border_size}px solid {_border_color};
        }}
        QFrame {{ 
            color: {_text_color};
            font: {_text_font};
        }}
    """
    toolTipStyle = """ 
        QLabel {{		
            background-color: {_dark_one};	
            color: {_text_foreground};
            padding-left: 10px;
            padding-right: 10px;
            network-radius: 17px;
            network: 0px solid transparent;
            network-left: 3px solid {_context_color};
            network-right: 3px solid {_context_color};
            font: 800 9pt "Segoe UI";
        }}
        """

    messagebox = """
                QMessageBox {
                background-color: #272c36; 
            
                font-size: 16pt;
                border-radius: 50px;
                }
               QMessageBox QLabel#qt_msgbox_label { /* textLabel */
                color: #298DFF;
                background-color: transparent;
                min-width: 390px; /* textLabel设置最小宽度可以相应的改变QMessageBox的最小宽度 */
                min-height: 60px; /* textLabel和iconLabel高度保持一致 */
                max-height:60px;
            }
            QMessageBox QLabel#qt_msgboxex_icon_label { /* iconLabel */
                width: 40px;
                height: 60px; /* textLabel和iconLabel高度保持一致 */
            }

            QMessageBox QPushButton {
                border: 1px solid #298DFF;
                border-radius: 10px;
                background-color: #F2F2F2;
                color: #272c36;
                text-align:center;
                font-family: "Microsoft YaHei";
                font-size: 15pt;
                min-width: 70px;
                min-height: 0px;
                margin-right: 30px;
                margin-top: 0px;
            }
            QMessageBox QPushButton:hover {
                background-color: #298DFF;
                color: #F2F2F2;
            }
            QMessageBox QPushButton:pressed {
                background-color: #257FE6;
            }
            /*QMessageBox QDialogButtonBox#qt_msgbox_buttonbox { /* buttonBox */
            button-layout: 0;  }*/

        """
    messagebox1 = {"""
    
            QMessageBox {
            background-color: #F2F2F2; /* QMessageBox背景颜色 */
            }
            
           QMessageBox QLabel#qt_msgbox_label { /* textLabel */
                color: #298DFF;
                background-color: transparent;
                min-width: 240px; /* textLabel设置最小宽度可以相应的改变QMessageBox的最小宽度 */
                min-height: 40px; /* textLabel和iconLabel高度保持一致 */
            }
            
            /*QMessageBox QLabel#qt_msgboxex_icon_label { /* iconLabel */
                width: 40px;
                height: 40px; /* textLabel和iconLabel高度保持一致 */
            }
            */
            QMessageBox QPushButton { /* QMessageBox中的QPushButton样式 */
                border: 1px solid #298DFF;
                border-radius: 3px;
                background-color: #F2F2F2;
                color: #298DFF;
                font-family: "Microsoft YaHei";
                font-size: 10pt;
                min-width: 70px;
                min-height: 25px;
            }
            
            QMessageBox QPushButton:hover {
                background-color: #298DFF;
                color: #F2F2F2;
            }
            
            QMessageBox QPushButton:pressed {
                background-color: #257FE6;
            }
            
            /*QMessageBox QDialogButtonBox#qt_msgbox_buttonbox { /* buttonBox */
                button-layout: 0; /* 设置QPushButton布局好像没啥作用 *//* 
    
    """
                  }
    ComBoStyle = """
        QComboBox {
            color:rgb(255, 255, 255);
            background-color: rgb(27, 29, 35);
            border-radius: 5px;
            border: 1px solid rgb(33, 37, 43);
            padding: 5px;
            padding-left: 10px;
            border-radius: 8px;
           
            
            
        }
        QComboBox:hover{ border: 2px solid rgb(86, 138, 242);
        }
        QComboBox::drop-down {
            background-color:rgba(13,61,101,10);
            subcontrol-origin: padding;
            subcontrol-position: top right;
            border-left-width: 1px;
            border-left-color: rgba(60, 60, 54, 250);
            border-left-style: solid;
            border-top-right-radius: 3px;
            border-bottom-right-radius: 3px;
            background-position: center;
            background-repeat: no-reperat; 
         }
         
         
         QCheckBox {color: rgb(200, 200, 200);font: 19pt;}
         QCheckBox::indicator:checked{
         background: #556B2F;
          }
          /*
         QComboBox::model:checked{
         background: rgb(255,71,209);
          }*/
               
         
         QComboBox QAbstractItemView {
            color: rgb(86,138,242);
            background-color: rgb(33, 37, 43);
            padding: 2px;
            selection-background-color: rgb(39, 44, 54);
            font: 19pt;
        }
        #xitongmingchengBox QAbstractItemView {
            color: rgb(86,138,242);
            background-color: rgb(33, 37, 43);
            padding: 2px;
            selection-background-color: rgb(39, 44, 54);
            font: 9pt;
        }
        
         QComboBox QAbstractItemView::item {
             height:28px;
             padding: 0px;
             margin: 0px;
             color: rgb(200, 200, 180);
              selection-color:rgb(33, 37, 43);
              selection-background-color: rgb(39, 44, 54);
            }
            
            
            
        QComboBox QAbstractItemView::item:selected {
            color: rgb(0,0,0);
            background: rgb(13,61,101);
        }
        /*
        QComboBox QAbstractItemView :checked {
            color: rgb(255,0,0);
            background: rgb(255,61,101);
        }*/
        
                                       
        QComboBox  QToolTip{
        color: rgb(0, 0, 0);
        font: 11pt;
        background:  rgb(129,118, 42);
        
        width: 800px;
        border-radius: 50px;
        border-bottom-left-radius: 20px;
        border-bottom-right-radius: 20px;

        }  
         

        QComboBox QAbstractScrollArea QScrollBar:vertical {
            border: none;
            background: rgb(52,59,72);
            width: 10px;
            border-radius: 2px;
        }
        QComboBox QAbstractScrollArea QScrollBar::handle:vertical {
            background: #568af2;
            min-height: 25px;
            border-radius: 4px;
          
        }
        QComboBox QAbstractScrollArea QScrollBar::add-line:vertical {
            background: #343b48;
            height: 20px;
            border-bottom-left-radius: 4px;
            border-bottom-right-radius: 4px;
            subcontrol-position: bottom;
            subcontrol-origin: margin;
           
        }
        QComboBox QAbstractScrollArea QScrollBar::sub-line:vertical {
            border: none;
            background: #343b48;
            height: 20px;
            border-top-left-radius: 4px;
            border-top-right-radius: 4px;
            subcontrol-position: top;
            subcontrol-origin: margin;
           
        }
        QComboBox QAbstractScrollArea QScrollBar::up-arrow:vertical, QScrollBar::down-arrow:vertical {
            background: none;
           
        }
        QComboBox QAbstractScrollArea QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {
            background: none; 
        }
        """
    ComBoStyle1 = """
            QComboBox {
                color:rgb(255, 255, 255);
                background-color: rgb(27, 50, 50);
                border-radius: 5px;
                border: 1px solid rgb(33, 37, 43);
                padding: 5px;
                padding-left: 10px;
                border-radius: 8px;



            }
            QComboBox:hover{ border: 2px solid rgb(86, 138, 242);
            }
            QComboBox::drop-down {
                background-color:rgba(13,61,101,10);
                subcontrol-origin: padding;
                subcontrol-position: top right;
                border-left-width: 1px;
                border-left-color: rgba(60, 60, 54, 250);
                border-left-style: solid;
                border-top-right-radius: 3px;
                border-bottom-right-radius: 3px;
                background-position: center;
                background-repeat: no-reperat; 
             }


             QCheckBox {color: rgb(200, 200, 200);font: 19pt;}
             QCheckBox::indicator:checked{
             background: #556B2F;
              }
              /*
             QComboBox::model:checked{
             background: rgb(255,71,209);
              }*/


             QComboBox QAbstractItemView {
                color: rgb(86,138,242);
                background-color: rgb(33, 37, 43);
                padding: 2px;
                selection-background-color: rgb(39, 44, 54);
                font: 19pt;
            }
            #xitongmingchengBox QAbstractItemView {
                color: rgb(86,138,242);
                background-color: rgb(33, 37, 43);
                padding: 2px;
                selection-background-color: rgb(39, 44, 54);
                font: 9pt;
            }

             QComboBox QAbstractItemView::item {
                 height:28px;
                 padding: 0px;
                 margin: 0px;
                 color: rgb(200, 200, 180);
                  selection-color:rgb(33, 37, 43);
                  selection-background-color: rgb(39, 44, 54);
                }



            QComboBox QAbstractItemView::item:selected {
                color: rgb(0,0,0);
                background: rgb(13,61,101);
            }
            /*
            QComboBox QAbstractItemView :checked {
                color: rgb(255,0,0);
                background: rgb(255,61,101);
            }*/


            QComboBox  QToolTip{
            color: rgb(0, 0, 0);
            font: 11pt;
            background:  rgb(129,118, 42);

            width: 800px;
            border-radius: 50px;
            border-bottom-left-radius: 20px;
            border-bottom-right-radius: 20px;

            }  


            QComboBox QAbstractScrollArea QScrollBar:vertical {
                border: none;
                background: rgb(52,59,72);
                width: 10px;
                border-radius: 2px;
            }
            QComboBox QAbstractScrollArea QScrollBar::handle:vertical {
                background: #568af2;
                min-height: 25px;
                border-radius: 4px;

            }
            QComboBox QAbstractScrollArea QScrollBar::add-line:vertical {
                background: #343b48;
                height: 20px;
                border-bottom-left-radius: 4px;
                border-bottom-right-radius: 4px;
                subcontrol-position: bottom;
                subcontrol-origin: margin;

            }
            QComboBox QAbstractScrollArea QScrollBar::sub-line:vertical {
                border: none;
                background: #343b48;
                height: 20px;
                border-top-left-radius: 4px;
                border-top-right-radius: 4px;
                subcontrol-position: top;
                subcontrol-origin: margin;

            }
            QComboBox QAbstractScrollArea QScrollBar::up-arrow:vertical, QScrollBar::down-arrow:vertical {
                background: none;

            }
            QComboBox QAbstractScrollArea QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {
                background: none; 
            }
            """
    ComBoStylecankao = """
        QComboBox {
    /* 边框宽度,线条样式,颜色 */
    border:3px solid red;

    /* 倒角 */
    border-radius:8px;

    /* 内边框 */
    padding:1px 18px 1px 3px;

    min-width:100px;
}

QComboBox:editable {
    background:green;
}

/* 渐变色:从左到右,黑白渐变 */
QComboBox:!editable,QComboBox::drop-down:editable {
    background:qlineargradient(x1:0, y1:0, x2:1, y2:0,
                               stop:0 rgb(0,0,0), stop:1 rgb(255,255,255));
}

/* 当下拉框打开时,背景颜色渐变 */
QComboBox:!editable:on, QComboBox::drop-down:editable:on {
    background: qlineargradient(x1: 0, y1: 0, x2: 1, y2: 0,
                                stop: 0 #D3D3D3, stop: 0.4 #D8D8D8,
                                stop: 0.5 #DDDDDD, stop: 1.0 #BBBBBB);
}

/* 当下拉框打开时, 移动显示框文本位置*/
QComboBox:on {
    padding-top: 3px;
    padding-left: 4px;
}

/* 下拉按钮 */
QComboBox::drop-down {
    subcontrol-origin: padding;

    /* 按钮位置,右上角 */
    subcontrol-position: top right;

    /* 按钮宽度 */
    width: 25px;

    /* 一条边框线控制 */
    border-left-width: 3px;
    border-left-color: red;
    border-left-style: solid;

    /* 倒角 */
    border-top-right-radius: 3px;
    border-bottom-right-radius: 3px;
}

/* 下拉按钮图片 */
QComboBox::down-arrow {
    border-image: url(:/gui/image/png_icons/xiaolog.png);
}

/* 下拉按钮位移 */
QComboBox::down-arrow:on {
    top: 1px;
    left: 1px;
}

/* 下拉列表里的颜色 */
QComboBox QAbstractItemView {
    border: 2px solid darkgray;
    selection-background-color: green;
}

        """
    ComBoTEWrong = """
        QComboBox {
            background-color: rgb(27, 29, 35);
            border-radius: 5px;
            border: 2px solid rgb(33, 37, 43);
            padding: 5px;
            padding-left: 10px;
            border-radius: 8px;
        }
        QComboBox:editable{ color: rgb(189, 147, 249); 
        }
        QComboBox:hover{ border: 2px solid rgb(189, 147, 249);
        }
        QComboBox::drop-down {
            subcontrol-origin: padding;
            subcontrol-position: top right;
            border-left-width: 3px;
            border-left-color: rgba(39, 44, 54, 150);
            border-left-style: solid;
            border-top-right-radius: 3px;
            border-bottom-right-radius: 3px;
            background-position: center;
            background-repeat: no-reperat;
         }
        QComboBox QAbstractItemView {
            color: rgb(86,138,242);
            background-color: rgb(33, 37, 43);
            padding: 10px;
            selection-background-color: rgb(39, 44, 54);
        }
        QComboBox QAbstractScrollArea QScrollBar:vertical {
            border: none;
            background: rgb(52,59,72);
            width: 10px;
            border-radius: 0px;
        }
        QComboBox QAbstractScrollArea QScrollBar::handle:vertical {
            background: #568af2;
            min-height: 25px;
            border-radius: 4px;
        }
        QComboBox QAbstractScrollArea QScrollBar::add-line:vertical {
            background: #343b48;
            height: 20px;
            border-bottom-left-radius: 4px;
            border-bottom-right-radius: 4px;
            subcontrol-position: bottom;
            subcontrol-origin: margin;
        }
        QComboBox QAbstractScrollArea QScrollBar::sub-line:vertical {
            border: none;
            background: #343b48;
            height: 20px;
            border-top-left-radius: 4px;
            border-top-right-radius: 4px;
            subcontrol-position: top;
            subcontrol-origin: margin;
        }
        QComboBox QAbstractScrollArea QScrollBar::up-arrow:vertical, QScrollBar::down-arrow:vertical {
            background: none;
        }
        QComboBox QAbstractScrollArea QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {
            background: none;
        }
        """
    # combo红色版保留样式
    ComBo1 = """
        QComboBox {
            background-color: rgb(27, 29, 35);
            network-radius: 5px;
            network: 2px solid rgb(33, 37, 43);
            padding: 5px;
            padding-left: 10px;
            network-radius: 8px;
        }
        QComboBox:editable{ color: rgb(189, 147, 249); 
        }
        QComboBox:hover{ border: 2px solid rgb(86, 138, 242);
        }
        QComboBox::drop-down {
            subcontrol-origin: padding;
            subcontrol-position: top right;
            network-left-width: 3px;
            network-left-color: rgba(39, 44, 54, 150);
            network-left-style: solid;
            network-top-right-radius: 3px;
            network-bottom-right-radius: 3px;
            background-position: center;
            background-repeat: no-reperat;
         }
        QComboBox QAbstractItemView {
            color: rgb(86,138,242);
            background-color: rgb(33, 37, 43);
            padding: 10px;
            selection-background-color: rgb(39, 44, 54);
        }
        
                                                
                                                

        QComboBox QAbstractScrollArea QScrollBar:vertical {
            border: none;
            background: rgb(52,59,72);
            width: 10px;
            border-radius: 0px;
        }
        QComboBox QAbstractScrollArea QScrollBar::handle:vertical {
            background: #568af2;
            min-height: 25px;
            border-radius: 4px;
        }
        QComboBox QAbstractScrollArea QScrollBar::add-line:vertical {
            background: #343b48;
            height: 20px;
            border-bottom-left-radius: 4px;
            border-bottom-right-radius: 4px;
            subcontrol-position: bottom;
            subcontrol-origin: margin;
        }
        QComboBox QAbstractScrollArea QScrollBar::sub-line:vertical {
            border: none;
            background: #343b48;
            height: 20px;
            border-top-left-radius: 4px;
            border-top-right-radius: 4px;
            subcontrol-position: top;
            subcontrol-origin: margin;
        }
        QComboBox QAbstractScrollArea QScrollBar::up-arrow:vertical, QScrollBar::down-arrow:vertical {
            background: none;
        }
        QComboBox QAbstractScrollArea QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {
            background: none;
        }
        """
    TableStyle = '''
        QTableWidget {{	
            background-color: {_bg_color};
            padding: 2px;
            border-radius: {_radius}px;
            gridline-color: {_grid_line_color};
            color: rgb(0,0,0);
            font: 13pt;

        }}
        QTableWidget::item{{
            border-color: none;
            padding-left: 2px;
            padding-right: 2px;
            gridline-color: rgb(44, 49, 60);
            border-bottom: 1px solid {_bottom_line_color};
        }}
        QTableWidget::item:selected{{
            background-color: {_selection_color};
            color: rgb(160,160,160);
        }}
        QHeaderView::section{{
            background-color: rgb(86,138,242);
            max-width: 30px;
            border: 1px solid rgb(44, 49, 58);
            border-style: none;
            border-bottom: 1px solid rgb(44, 49, 60);
            border-right: 1px solid rgb(44, 49, 60);
        }}
        QHeaderView::section:hover{{color: rgb(86,138,242);}}
        QTableWidget::horizontalHeader {{	
            background-color: rgb(33, 37, 43);
        }}
        QTableWidget QTableCornerButton::section {{
            border: none;
            background-color: {_header_horizontal_color};
            padding: 3px;
            border-top-left-radius: {_radius}px;
        }}
        QHeaderView::section:horizontal
        {{
            border: none;
            background-color: {_header_horizontal_color};
            padding: 3px;
            font: 14pt;
        }}
        QHeaderView::section:vertical
        {{
            border: none;
            background-color: {_header_vertical_color};
            padding-left: 5px;
            padding-right: 5px;
            border-bottom: 1px solid {_bottom_line_color};
            margin-bottom: 1px;
        }}
        QScrollBar:horizontal {{
            border: none;
            background: {_scroll_bar_bg_color};
            height: 10px;
            margin: 0px 21px 0px 21px;
            border-radius: 0px;
        }}
        QScrollBar::handle:horizontal {{
            background: {_context_color};
            min-width: 25px;
            border-radius: 4px
        }}
        QScrollBar::add-line:horizontal {{
            border: none;
            background: {_scroll_bar_btn_color};
            width: 20px;
            border-top-right-radius: 4px;
            border-bottom-right-radius: 4px;
            subcontrol-position: right;
            subcontrol-origin: margin;
        }}
        QScrollBar::sub-line:horizontal {{
            border: none;
            background: {_scroll_bar_btn_color};
            width: 20px;
            border-top-left-radius: 4px;
            border-bottom-left-radius: 4px;
            subcontrol-position: left;
            subcontrol-origin: margin;
        }}
        QScrollBar::up-arrow:horizontal, QScrollBar::down-arrow:horizontal
        {{
             background: none;
        }}
        QScrollBar::add-page:horizontal, QScrollBar::sub-page:horizontal
        {{
             background: none;
        }}
        QScrollBar:vertical {{
            border: none;
            background: {_scroll_bar_bg_color};
            width: 10px;
            margin: 21px 0px 21px 0px;
            border-radius: 0px;
        }}
        QScrollBar::handle:vertical {{	
            background: {_context_color};
            min-height: 25px;
            border-radius: 4px
        }}
        QScrollBar::add-line:vertical {{
            border: none;
            background: {_scroll_bar_btn_color};
            height: 20px;
            border-bottom-left-radius: 4px;
            border-bottom-right-radius: 4px;
            subcontrol-position: bottom;
            subcontrol-origin: margin;
        }}
        QScrollBar::sub-line:vertical {{
            border: none;
            background: {_scroll_bar_btn_color};
            height: 20px;
            border-top-left-radius: 4px;
            border-top-right-radius: 4px;
            subcontrol-position: top;
            subcontrol-origin: margin;
        }}
        QScrollBar::up-arrow:vertical, QScrollBar::down-arrow:vertical {{
             background: none;
        }}

        QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {{
             background: none;
        }}
    '''
    MenuStyle = '''
        QMenu{
            background-color: rgb(27, 29, 35);
            network-radius: 8px;
            network: 1px solid rgb(33, 37, 43);
            padding-left: 3px;
            width: 190px;
        }
        QMenu::item {
            color: rgb(195,204,223);
            background-color: rgb(33, 37, 43);
            padding: 10px;
            selection-background-color: rgb(39, 44, 54);
            network: 1px solid rgb(27, 29, 352);
            width: 160px;
            font: 13pt;
        }
        QMenu::item:selected{
            color: rgb(86, 138, 242);
        }
    '''
    # TODO 样式优化一下
    TabStyle = '''
        QTabWidget::pane {
            background-color: transparent;
            padding: 2px;
            border-radius: 5px;
            border-bottom: 1px solid rgb(44, 49, 60);
        }
        QTabBar::tab {
            background-color: rgb(33, 37, 43);
            width: 150;
            border-radius: 3px;
        }
        QTabBar::tab:hover{color: rgb(86,138,242);}
        QTabBar::tab:selected {
            border-bottom: 2px solid #568af2;
            color: rgb(86,138,242);
        }
    '''
    RadioBtnStyle = '''
        QRadioButton::indicator{
            network: 3px solid rgb(52, 59, 72);
            width: 15px;
            height: 15px;
            network-radius: 10px;
            background: rgb(44, 49, 60);
        }
        QRadioButton::indicator:hover{ network: 3px solid rgb(86, 138, 242);}
        QRadioButton::indicator:checked{
            background: 3px solid rgb(108,153,244);
            network: 3px solid rgb(52, 59, 72);
        }
    '''
    SAVerticalStyle = '''
        QScrollBar:vertical {
            border: none;
            background: rgb(52,59,72);
            width: 12px;
            border-radius: 0px;
        }
        QScrollBar::handle:vertical {
            background: #568af2;
            min-height: 25px;
            border-radius: 4px;
        }
        QScrollBar::add-line:vertical {
            background: #343b48;
            height: 20px;
            border-bottom-left-radius: 4px;
            border-bottom-right-radius: 4px;
            subcontrol-position: bottom;
            subcontrol-origin: margin;
        }
        QScrollBar::sub-line:vertical {
            border: none;
            background: #343b48;
            height: 20px;
            border-top-left-radius: 4px;
            border-top-right-radius: 4px;
            subcontrol-position: top;
            subcontrol-origin: margin;
        }
        QScrollBar::up-arrow:vertical, QScrollBar::down-arrow:vertical {
            background: none;
        }
        QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {
            background: none;
        }  
    '''
    SAHorizontalStyle = '''
        QScrollBar:horizontal {
            border: none;
            background: rgb(52,59,72);
            height: 12px;
            border-radius: 0px;
        }
        QScrollBar::handle:horizontal {
            background: #568af2;
            min-width: 25px;
            border-radius: 4px;
        }
        QScrollBar::add-line:horizontal {
            background: #343b48;
            width: 20px;
            border-right-top-radius: 4px;
            border-right-bottom-radius: 4px;
            subcontrol-position: right;
            subcontrol-origin: margin;
        }
        QScrollBar::sub-line:horizontal {
            border: none;
            background: #343b48;
            width: 20px;
            border-left-top-radius: 4px;
            border-left-bottom-radius: 4px;
            subcontrol-position: left;
            subcontrol-origin: margin;
        }
        QScrollBar::up-arrow:horizontal, QScrollBar::down-arrow:horizontal {
            background: none;
        }
        QScrollBar::add-page:horizontal, QScrollBar::sub-page:horizontal {
            background: none;
        }
    '''
    LabelStyle = '''
        QToolTip{
            font-size:10px;
        }
    '''



'''
    color_HSV
'''

from PyQtGuiLib.header import (
    QPixmap,
    QPainter,
    Signal,
    qt,
    QLinearGradient,
    QConicalGradient,
    QColor,
    QMouseEvent,
    QPaintEvent,
    QPoint,
    QWidget,
    QBrush,
    QVBoxLayout,
    QRect,
    QPen
)

# 移动圆圈类
class MousePoint:
    def __init__(self,x=0,y=0,r=0):
        self.__x = x
        self.__y = y
        self.__r = r

    def size(self,x:int=None,y:int=None):
        if x is None or y is None:
            return self.x(), self.y()
        else:
            self.x(x), self.y(y)

    def x(self,v:int=None):
        if v is None:
            return self.__x
        else:
            self.__x = v

    def y(self,v:int=None):
        if v is None:
            return self.__y
        else:
            self.__y = v

    def r(self,v:int=None):
        if v is None:
            return self.__r
        else:
            self.__x = v

    def drawImage(self):
        pass

# 所有调色图像的抽象
class ColorABC(QWidget):
    # 返回颜色rgba
    '''
    rgbaChange:返回(r,g,b,a)
    hsvChange:返回(h,s,v,a)
    nameChange:返回 #xxxxxx
    '''
    rgbaChange = Signal(tuple)
    hsvChange = Signal(tuple)
    nameChange = Signal(str)

    def __init__(self,*args,**kwargs):
        # 移动点的大小
        self._mouse = MousePoint(0, 0, 5)

        super().__init__(*args,**kwargs)

    def setR(self,r):
        self._mouse.r(r)

    def _update(self,pos:QPoint):
        x,y = pos.x(),pos.y()
        if (x >= 0 and x <= self.width() - self._mouse.r() and
            y >= 0 and y <= self.height() - self._mouse.r()):
            self._mouse.size(x,y)
            #
            self.sendSignal(x,y)
            self.update()

    # 发送信号
    def sendSignal(self,x,y):
        color = self.grab(QRect(x, y, 1, 1)).toImage().pixelColor(0, 0)
        # print(color.getRgb())
        self.hsvChange.emit(color.getHsv())
        self.rgbaChange.emit(color.getRgb())
        self.nameChange.emit(color.name())

    # 留给子类重写的部分
    def __painter__(self,painter:QPainter):
        pass

    def mousePressEvent(self, e: QMouseEvent) -> None:
        self._update(e.pos())

    def mouseMoveEvent(self, e: QMouseEvent) -> None:
        self._update(e.pos())

    def paintEvent(self, event:QPaintEvent) -> None:
        painter = QPainter(self)
        painter.setRenderHints(qt.Antialiasing | qt.SmoothPixmapTransform | qt.TextAntialiasing)

        self.__painter__(painter)
        p = QPen()
        p.setWidth(2)
        painter.setPen(p)
        painter.drawEllipse(*self._mouse.size(), self._mouse.r() * 2, self._mouse.r() * 2)
        painter.end()


class ColorHsv(ColorABC):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        self.__radius = 5
        self.setFixedHeight(40)
        self.setUI()

    def setRadius(self,r:int):
        self.__radius = r

    def setUI(self):
        self.pix = QPixmap(self.size())
        self.pix.fill(qt.transparent)
        self.createHuePixmap()

    def __painter__(self,painter:QPainter):
        painter.setPen(qt.white)
        painter.drawPixmap(self.rect(),self.pix)

    # 创建HSV颜色条
    def createHuePixmap(self):
        painter = QPainter(self.pix)
        painter.setRenderHints(qt.Antialiasing | qt.SmoothPixmapTransform)  # 抗锯齿
        gradient = QLinearGradient(self.width(), 0, 0, 0)

        i = 0.0
        gradient.setColorAt(0, QColor.fromHsvF(0, 1, 1, 1))
        while i < 1.0:
            gradient.setColorAt(i, QColor.fromHsvF(i, 1, 1, 1))
            i += 1.0 / 16.0
        gradient.setColorAt(1, QColor.fromHsvF(0, 1, 1, 1))
        painter.setPen(qt.NoPen)
        painter.setBrush(gradient)
        painter.drawRoundedRect(self.rect(),self.__radius,self.__radius)


class ColorLump(ColorABC):
    def __init__(self,*args,**kwargs):

        super().__init__(*args,**kwargs)
        self.__bgcolor = QColor(0, 255, 0, 255)

        # 鼠标是否过半
        self.is_half = False
        self.setUI()

    def setAlpha(self,a:int):
        self.__bgcolor.setAlpha(a)
        self.colorLayer()

    def setBgColor(self,color:QColor):
        self.__bgcolor = color
        self.colorLayer()

    def bgColor(self) -> QColor:
        return self.__bgcolor

    def setUI(self):
        self.grayLayer()
        self.colorLayer()

    def mouseMoveEvent(self, e: QMouseEvent) -> None:
        super().mouseMoveEvent(e)
        # 过半判断
        self.is_half = True if(e.y() > self.height()//2) else False

    # 灰色图层
    def grayLayer(self):
        self.gray_pix = QPixmap(self.size())
        self.gray_pix.fill(qt.transparent)
        self.createGrayPixmap()

    # 彩色图层
    def colorLayer(self):
        self.pix = QPixmap(self.size())
        self.pix.fill(qt.transparent)
        self.createPixmap()

    def __painter__(self,painter:QPainter):
        painter.drawPixmap(self.rect(),self.pix)
        if self.is_half:
            painter.setPen(qt.white)
        painter.drawPixmap(self.rect(), self.gray_pix)

    def createGrayPixmap(self):
        painter = QPainter(self.gray_pix)
        painter.setRenderHints(qt.Antialiasing)

        gradient = QLinearGradient(0, 0, 0, self.height())
        gradient.setColorAt(0, QColor(0, 0, 0, 0))
        gradient.setColorAt(1, QColor("#000"))
        painter.setPen(qt.NoPen)
        painter.setBrush(gradient)
        painter.drawRect(self.rect())

    def createPixmap(self):
        painter = QPainter(self.pix)
        painter.setRenderHints(qt.Antialiasing)

        gradient = QLinearGradient(0, 0, self.width(), 0)
        gradient.setColorAt(0, QColor("#fff"))

        gradient.setColorAt(1, self.bgColor())

        painter.setPen(qt.NoPen)
        painter.setBrush(gradient)

        painter.drawRect(self.rect())

# -----------------------------

# 矩形
class ColorRect(QWidget):
    # 返回颜色rgba
    rgbaChange = Signal(tuple)
    # 返回hsv 中的h (h表示色调)
    hsvChange = Signal(int)
    nameChange = Signal(str)

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(256,256)

        self.setMouseTracking(False)
        self.Init()
        self.myEvent()

    def Init(self):
        self.vlayout = QVBoxLayout(self)
        self.vlayout.setSpacing(1)
        self.vlayout.setContentsMargins(0,0,0,0)

        self.color_hsv = ColorHsv()
        self.color_lump = ColorLump()
        self.vlayout.addWidget(self.color_hsv)
        self.vlayout.addWidget(self.color_lump)

    def setAlpha(self,a:int):
        self.color_lump.setAlpha(a)

    def myEvent(self):
        self.color_hsv.rgbaChange.connect(self.change_lump_event)
        self.color_hsv.hsvChange.connect(self.hsvChange.emit)
        self.color_hsv.nameChange.connect(self.nameChange.emit)
        self.color_lump.rgbaChange.connect(self.rgbaChange.emit)
        self.color_lump.hsvChange.connect(self.hsvChange.emit)
        self.color_lump.nameChange.connect(self.nameChange.emit)

    def change_lump_event(self,rgba:tuple):
        self.rgbaChange.emit(rgba)
        # ----
        self.color_lump.setBgColor(QColor(*rgba))
        self.update()


# 色轮
class ColorWheel(ColorABC):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(240,240)

        # 如果亮度过低,则改变圆圈颜色
        self.is_alpha = False

        self.colors = [(0,QColor(255, 0, 0,255)),
                       (60,QColor(255, 255, 0,255)),
                       (120,QColor(0, 255, 0,255)),
                       (180,QColor(0,255,255,255)),
                       (240,QColor(0, 0, 255,255)),
                       (300,QColor(255, 0, 255,255)),
                       (360,QColor(255, 0, 0,255))
                       ]

        self.setAttribute(qt.WA_TranslucentBackground)
        self.setWindowFlags(qt.FramelessWindowHint | qt.Widget)

        self.setUI()

    def setAlpha(self,a:int):
        for _,color in self.colors:
            color.setAlpha(a)

        self.is_alpha = True if(a <= 220) else False

        self.createColorWheel()

    def setUI(self):
        # 底色
        self.bgColorWheel()
        self.createColorWheel()

    # 色轮底色
    def bgColorWheel(self):
        self.bgpix = QPixmap(self.size())
        self.bgpix.fill(qt.transparent)

        painter = QPainter(self.bgpix)
        painter.setRenderHints(qt.Antialiasing)

        gradient = QConicalGradient(self.width() // 2, self.height() // 2, 6)
        color = QColor(0,0,0,255)
        for v in range(0,420,60):
            gradient.setColorAt(v / 360, color)

        bru = QBrush(gradient)
        painter.setPen(qt.NoPen)
        painter.setBrush(bru)
        painter.drawEllipse(self.rect())

    def createColorWheel(self):
        self.pix = QPixmap(self.size())
        self.pix.fill(qt.transparent)

        painter = QPainter(self.pix)
        gradient = QConicalGradient(self.width() // 2, self.height() // 2, 6)
        painter.setRenderHints(qt.Antialiasing)

        # 绘制
        for v, color in self.colors:
            gradient.setColorAt(v / 360, color)

        bru = QBrush(gradient)
        painter.setPen(qt.NoPen)
        painter.setBrush(bru)
        painter.drawEllipse(self.rect())

        painter.setBrush(QBrush(qt.transparent))
        painter.drawEllipse(20, 20, 200, 255)
        painter.setPen(qt.white)


    def __painter__(self,painter:QPainter):
        painter.drawPixmap(self.rect(), self.bgpix)
        if self.is_alpha:
            painter.setPen(qt.white)
        painter.drawPixmap(self.rect(), self.pix)


import sys
from PyQt5.QtGui import QPainter, QBrush, QColor
from PyQt5.QtCore import QTimer, QPointF, Qt
from PyQt5.QtWidgets import QApplication, QWidget
import random

class Particle:
    def __init__(self, pos):
        self.pos = pos
        self.speed = QPointF((0.5 - random.random()) * 2, (0.5 - random.random()) * 2)

    def update(self):
        self.pos += self.speed


class ParticleSystem:
    def __init__(self, num_particles):
        self.particles = [Particle(QPointF(random.random() * 400, random.random() * 400)) for _ in range(num_particles)]

    def update(self):
        for particle in self.particles:
            particle.update()

    def draw(self, painter):
        brush = QBrush(QColor(255, 255, 255))
        for particle in self.particles:
            painter.setBrush(brush)
            painter.drawEllipse(particle.pos, 2, 2)


class Widget(QWidget):
    def __init__(self):
        super().__init__()

        self.setGeometry(100, 100, 400, 400)

        self.particle_system = ParticleSystem(20000)

        self.timer = QTimer(self)
        self.timer.timeout.connect(self.update_particles)
        self.timer.start(10)

    def update_particles(self):
        self.particle_system.update()
        self.update()

    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)
        self.particle_system.draw(painter)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    w = Widget()
    w.show()
    sys.exit(app.exec_())



# -*- coding:utf-8 -*-
# @time:2023/2/813:18
# @author:LX
# @file:styleAnalysis.py
# @software:PyCharm

'''

    QSS 样式解析器
'''
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    SyntaxError,
    KeyError,
    QMutex
)
import typing
import re

Ang = typing.TypeVar("Any",str,int)


def dictTostr(qss_dict) -> str:
    combination = ""
    for selector, attribute in qss_dict.items():
        combination += selector + "{\n"
        for attrk, attrv in attribute.items():
            combination += "%s:%s;\n" % (attrk, attrv)
        combination += "}\n"
    return combination


class Qss:
    def __init__(self,qss:str,qs=None,parent=None):
        self.__parent =parent
        self.__qs = qs

        self._qss_str = self.uncomment(qss)
        self._qss_dict = dict()
        self._qss_header = ""
        self._qss_body = ""

        self.Init()

    # Uncomment information
    def uncomment(self,qss:str) -> str:
        return re.sub("/\*.*?\*/","",qss,flags=re.DOTALL)

    def Init(self):
        header = re.findall(".*{", self._qss_str, re.DOTALL)
        if header:
            self._qss_header = header[0].replace("\n", "").replace("{", "").strip()
        else:
            raise SyntaxError("Syntax error, missing {")

        # ---
        body = re.findall(r"{(.*)}", self._qss_str, re.DOTALL)
        if body:
            self._qss_body = body[0].strip()
        self._qss_dict = {self.header(): self.bodyToDict()}

    def header(self)->str:
        return self._qss_header

    def headerSubdivision(self)->list:
        if "," in self.header():
            return self.header().split(",")
        return self.header().split(" ")

    def body(self)->str:
        return self._qss_body

    def bodySubdivision(self)->list:
        bodysub_list = []

        for v in self.body().split(";"):
            if v:
                bodysub_list.append(v.strip())
        return bodysub_list

    def bodyToDict(self)->dict:
        s_dict = dict()
        for v in self.bodySubdivision():
            key,value = v.split(":",1)
            s_dict[key]=value.strip()
        return s_dict

    def toDict(self)->dict:
        return self._qss_dict

    def attr(self,key:str)->str:
        if key in self.bodyToDict():
            return self.bodyToDict()[key]
        else:
           raise KeyError("Without this attribute,'%s'" % key)

    def updateAttr(self,key:str,value:str):
        self._qss_dict[self.header()][key]=value
        self._qss_str = dictTostr(self._qss_dict)
        self.Init()
        if self.__parent is not None:
            self.__qs.updateStyleSheet()

    def removeAttr(self,key:str):
        if key in self._qss_dict[self.header()]:
            del self._qss_dict[self.header()][key]
            self._qss_str = dictTostr(self._qss_dict)
            self.Init()
            if self.__parent is not None:
                self.__qs.updateStyleSheet()
        else:
            raise TypeError("Without this attribute,'%s'"%key)

    def isAttr(self,key:str)->bool:
        if self._qss_dict[self.header()].get(key):
            return True
        return False

    def toStr(self)->str:
        return self._qss_str

    def __str__(self):
        return self._qss_str


class QssStyleAnalysis:
    def __init__(self,parent=None):
        self.__parent = parent
        self._qss = [] # type:[Qss]
        # map
        self._map_qss = dict()
        self._reverse_map_qss = dict()

        self.__mutex = QMutex()  # Thread lock

    def setParent(self,parent):
        self.__parent = parent

    def parent(self):
        return self.__parent

    # Decompose multiple groups of QSS
    def groupDecomposition(self,styles):
        re_styles=re.findall(r".*?}", styles, re.DOTALL)
        if not re_styles: #
            '''
                Deal with special circumstances:
                        color: rgb(0, 255, 127);
                        background-color:rgb(0, 170, 0);
            '''
            str_p = str(type(self.__parent))
            if "QtWidgets" in str_p:
                head = re.findall("QtWidgets\.(.*)\'",str_p)
            else:
                head = re.findall("\.(.*)\'",str_p)

            re_styles = [head[0]+"{"+styles+"}"]

        return [v.strip() for v in re_styles]

    def count(self) -> int:
        return len(self._qss)

    def setQSS(self,qss:str):
        # Preprocessing qss
        self._qss = [Qss(qss,self,self.__parent) for qss in self.groupDecomposition(qss)]
        # Mapping coordinate
        self.__mappCoordinate(0,self.count())

        if self.__parent is not None:
            self.__updateStyle(self.__parent)

    # Bidirectional mapping
    def __mappCoordinate(self,s,e):
        for i in range(s,e):
            self._map_qss[self.selectorIndex(i).header()] = i
            self._reverse_map_qss[str(i)] = self.selectorIndex(i).header()

    # Inherits styles that have been set elsewhere
    def inherit(self):
        self.setQSS(self.__parent.styleSheet())

    def setQSSDict(self,qss_dict:dict):
        self.setQSS(dictTostr(qss_dict))

    def appendQSS(self,qss:str):
        new_qss = [Qss(qss,self,self.__parent) for qss in self.groupDecomposition(qss)]
        '''
            Handle attribute fusion when the appended attribute conflicts with the original attribute
        '''
        # temp_hear = []
        for qss in new_qss:
            if self.isSelectKey(qss.header()):
                for attr,value in self.toDict()[qss.header()].items():
                    if qss.isAttr(attr) is False:
                        qss.updateAttr(attr, value)

        # del temp_hear
        old_count = self.count()
        self._qss.extend(new_qss)

        # remap
        self.__mappCoordinate(old_count,self.count())
        if self.__parent is not None:
            self.__updateStyle(self.__parent)

    def appendQSSDict(self,qss_dict:dict):
        self.appendQSS(dictTostr(qss_dict))

    def selectorKey(self,key:str)->Qss:
        return self.selectorIndex(self._map_qss[key])

    def selectorIndex(self,i:int)->Qss:
        return self._qss[i]

    def selector(self,ang:Ang)->Qss:
        if isinstance(ang,int):
            return self.selectorIndex(ang)
        elif isinstance(ang,str):
            return self.selectorKey(ang)
        else:
            raise TypeError("Parameter error!")

    def removeSelectorIndex(self,index:int):
        self._qss.remove(self._qss[index])
        self.updateStyleSheet()
        select_name = self._reverse_map_qss[str(index)]
        del self._map_qss[select_name]
        del self._reverse_map_qss[str(index)]

        # Rebidirectional mapping
        self.__mappCoordinate(0,self.count())

    def removeSelectorKey(self,key:str):
        self.removeSelectorIndex(self._map_qss[key])

    def removeSelector(self,ang:Ang):
        if isinstance(ang, int):
            return self.removeSelectorIndex(ang)
        elif isinstance(ang, str):
            return self.removeSelectorKey(ang)
        else:
            raise TypeError("Parameter error!")

    def isSelectKey(self,key:str):
        if self._map_qss.get(key,-1) >= 0:
            return True
        return False

    def toDict(self) -> dict:
        qss_dict = dict()
        for i in range(self.count()):
            qss_dict.update(self.selectorIndex(i).toDict())
        return qss_dict

    def header(self) -> list:
        return list(self.toDict().keys())

    def toStr(self) -> str:
        return dictTostr(self.toDict())

    def __updateStyle(self, parent):
        # Thread safety
        self.__mutex.lock()
        parent.setStyleSheet("")
        parent.setStyleSheet(self.toStr())
        if PYQT_VERSIONS == "PyQt5":
            parent.update()
        else:
            parent.repaint()
        self.__mutex.unlock()

    # updata QSS
    def updateStyleSheet(self,ang=None,parent=None):
        if ang is None:
            ang = self._qss[0].header()
        if parent is None and self.__parent is None:
            raise TypeError("Cannot update without a parent class!")
        elif parent:
            pass
            # self.selector(ang)
        elif self.__parent:
            parent = self.__parent
            # self.selector(ang)

        self.__updateStyle(parent)

    def __str__(self):
        return self.toStr()